#production.py
#A set of routines for building a set of production files for fabricating and assembling
#circuit boards

import os.path
import zipfile
from pymomo.exceptions import *
from pymomo.utilities.paths import MomoPaths
from datetime import date
import platform
import subprocess

class ProductionFileGenerator:
	"""
	Helpful routines for building production files for PCB fabrication and assembly
	"""

	def __init__(self, board):
		config = MomoPaths().config
		
		template_file = os.path.join(config, 'pcb', board.fab_engine + ".json")
		self.template = self._load_template(template_file, board.fab_template)
		self.board = board

	def save_readme(self, path):
		"""
		Create a README file for this board based on the information it contains
		and the template for the production files.
		"""

		basename = self._basename()

		with open(path, "w") as f:
			f.write("PCB Fabrication Files\n")
			f.write("%s\n" % self.board.company)
			f.write("Name: %s\n" % self.board.part)
			f.write("Revision: %s\n" % self.board.revision)
			f.write("Dimensions: %sx%s %s\n" % (self.board.width, self.board.height, self.board.units))
			f.write("Date Created: %s\n\n" % str(date.today()))
			f.write("Folder Contents:\n")

			for name, layer in self.template['layers'].iteritems():
				f.write("%s.%s: %s\n" % (basename, layer['extension'], name))

	def _build_production(self, outdir, layer):
		basename = self._basename()
		path = os.path.join(outdir, "%s.%s" %(basename, layer['extension']))


		self.board.board.build_production_file(path, layer['program_layers'], layer['type'])

		if 'remove' in layer:
			remname = "%s.%s" % (basename, layer['remove'])
			os.remove(os.path.join(outdir, remname))

	def build_fab(self, fab_dir):
		"""
		Create production Gerber and Excellon files for pcb fabrication
		"""

		self._ensure_dir_exists(fab_dir)
		self.save_readme(os.path.join(fab_dir, 'README.txt'))
		
		for name, layer in self.template['layers'].iteritems():
			self._build_production(fab_dir, layer)

	def build_assembly(self, ass_dir):
		self._ensure_dir_exists(ass_dir) 
		self._build_production(ass_dir, self.template['assembly'])

	def build_production(self, variant, output_dir):
		basename = self._basename()

		fab_dir = os.path.join(output_dir, 'fabrication')
		ass_dir = os.path.join(output_dir, 'assembly')

		#Create fabrication files
		self.build_fab(fab_dir)
		self.zipfab(fab_dir, os.path.join(output_dir, basename + '_fab'))

		#Create assembly files
		self.build_assembly(ass_dir)

		#Create BOM
		bompath = os.path.join(ass_dir, "BOM_%s.xlsx" % basename)
		self.board.export_bom(bompath, variant, format='excel')

	def _basename(self):
		return self.board.part.replace(' ', '_').lower()

	def _ensure_dir_exists(self, output_dir):
		if not os.path.isdir(output_dir):
			os.makedirs(output_dir)

	def _load_template(self, templatefile, template_name):
		"""
		Load a list of possible templates for making gerbers from a json file
		"""

		import json
		with open(templatefile, "r") as f:
			templates = json.load(f)

		if "templates" not in templates:
			raise DataError("Unknown file format for gerber generation template file", path=templatefile, reason="did not contain a 'templates' key")

		options = templates['templates']

		if template_name not in options:
			raise ArgumentError("Unknown template specified for gerber generation", possible_names=options.keys(), name=template_name)

		return options[template_name]

	def zipfab(self, path, output):
		"""
		Create a zipfile of the direction path with the name output.zip that will expand into a directory 
		with the same name as output containing all of the files in path. 
		"""

		zip = zipfile.ZipFile(output+'.zip', 'w', zipfile.ZIP_DEFLATED)
		for root, dirs, files in os.walk(path):
			for file in files:
				zip.write(os.path.join(root, file), os.path.join(os.path.basename(output), file))

		zip.close()



import SCons.Builder

import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..'))
from pymomo.hex8 import symbols

def merge_sym(target, source, env):
	"""
	Given two source files, the first being the mib12_executive and the second
	being an application hex, merge the two into a complete application hex
	"""

	first = symbols.XC8SymbolTable(str(source[0]))

	first.merge(str(source[1]))
	first.generate_stb_file(str(target[0]))

_merge_sym = SCons.Builder.Builder(
	action = merge_sym
	)

def generate(env):
	env['BUILDERS']['merge_mib12_symbols'] = _merge_sym

def exists(env):
	return 1




import sys
from pymomo.utilities.typedargs import type_system

class ProgressBar:
    """
    A simple progress bar that updates itself in the console.

    If the program is being run in interactive mode, display the progress_bar
    otherwise do not display it
    """

    def __init__(self, title, count=100):
        self.title = title
        self.prog = 0 
        self.count = count

    def start(self):
        if type_system.interactive:
            sys.stdout.write(self.title + ": [" + "-"*40 + "]" + chr(8)*41)
            sys.stdout.flush()

        self.prog = 0

    def progress(self, x):
        x = int(x * 40 // self.count)

        if type_system.interactive:
            sys.stdout.write("#" * (x - self.prog))
            sys.stdout.flush()

        self.prog = x

    def end(self):
        if type_system.interactive:
            sys.stdout.write("#" * (40 - self.prog) + "]\n")
            sys.stdout.flush()



import unittest
import os.path
from nose.tools import *
from pymomo.exceptions import *
from pymomo.pcb.eagle import part
from pymomo.pcb import CircuitBoard
import tempfile
import shutil
import os

def test_parse_distpn():
	#Board is missing Company and Dimension unit attributes
	board1 = os.path.join(os.path.dirname(__file__), 'eagle', 'controller_missing_attrs.brd')
	board2 = os.path.join(os.path.dirname(__file__), 'eagle', 'controller_dist_distpn.brd')
	b1 = CircuitBoard(board1)
	b2 = CircuitBoard(board2)

	p1 = b1.find('U5')
	p2 = b2.find('U5')

	print str(p1)
	print str(p2)

	assert p1 == p2

def test_assy_vars():
	board = os.path.join(os.path.dirname(__file__), 'eagle', 'assyvars.brd')
	b2 = CircuitBoard(board)
	variants = b2.get_variants()
	print variants

	assert len(variants) == 3
	assert 'VAR1' in variants
	assert 'VAR2' in variants
	assert 'VAR3' in variants

	p1 = b2.find('R1', 'VAR1')
	p2 = b2.find('R1', 'VAR2')

	print p1
	print p2

	assert p1.value == '1K'
	assert p2.value == '25k'

	assert p1.dist == 'Digikey'
	assert p2.dist == 'Digikey'

	assert p1.distpn == '1'
	assert p2.distpn == '2'

@raises(ArgumentError)
def test_assy_nopop():
	board = os.path.join(os.path.dirname(__file__), 'eagle', 'assyvars.brd')
	b2 = CircuitBoard(board)

	b2.find('R1', 'VAR3')

def test_nopop_list():
	board = os.path.join(os.path.dirname(__file__), 'eagle', 'assyvars.brd')
	b2 = CircuitBoard(board)

	p1 = b2.nonpopulated_parts('VAR1')
	p2 = b2.nonpopulated_parts('VAR2')
	p3 = b2.nonpopulated_parts('VAR3')

	assert len(p1) == 0
	assert len(p2) == 0
	assert len(p3) == 1

	assert p3[0] == 'R1'
 
def test_update_attribute():
	board = os.path.join(os.path.dirname(__file__), 'eagle', 'controller_missing_attrs.brd')
	brdcopy = _copy_to_temp(board)

	b = CircuitBoard(brdcopy)

	u5 = b.find('U5')
	assert u5.mpn == None
	assert u5.manu == None

	b.board.set_metadata(u5, None, 'mpn', "test_mpn")
	b.board.set_metadata(u5, None, 'manu', "test_manu")
	b.board.set_metadata(u5, None, 'digikey-pn', "test_digipn")
	bn = CircuitBoard(brdcopy)
	
	u5 = bn.find('U5')
	assert u5.mpn == 'TEST_MPN'
	assert u5.manu == 'TEST_MANU'
	assert u5.distpn == 'TEST_DIGIPN'

	os.remove(brdcopy)

def _copy_to_temp(src):
	dst = tempfile.NamedTemporaryFile(delete=False)

	with open(src, "r+b") as f:
		shutil.copyfileobj(f, dst)

	dst.close()

	return dst.name


#Automatic building of firmware and unit tests using the
#scons based momo build system

import utilities
import pic24
import pic12
import unit_test
import unit_test12
from SCons.Script import *
import os.path
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
from pymomo.exceptions import *
import pymomo

def autobuild_pic12(module, test_dir='test', modulefile=None):
	"""
	Build the given module for all targets and build all unit tests.
	targets are determined from /config/build_settings.json using
	the module name and the tests are found automatically and built 
	using their builtin metadata
	"""
	
	try:
		family = utilities.get_family('mib12', modulefile=modulefile)
		family.for_all_targets(module, lambda x: pic12.build_module(module, x))
		
		unit_test.build_units(test_dir, family.targets(module))

		Alias('release', os.path.join('build', 'output'))
		Alias('test', os.path.join('build', 'test', 'output'))
		Default('release')
	except MoMoException as e:
		print e.format()
		sys.exit(1)

def autobuild_pic24(module, test_dir='test', modulefile=None, postprocess_hex=None):
	"""
	Build the given pic24 module for all targets.
	"""

	try:
		family = utilities.get_family('mib24', modulefile=modulefile)
		family.for_all_targets(module, lambda x: pic24.build_module(module, x, postprocess_hex=postprocess_hex))

		Alias('release', os.path.join('build', 'output'))
		Alias('test', os.path.join('build', 'test', 'output'))
		Default('release')
	except MoMoException as e:
		print e.format()
		sys.exit(1)



from pymomo.commander.hwtest.fixtures import *
from pymomo.commander.exceptions import *
from intelhex import IntelHex16bit
import random
import os.path

class TestFirmwareCache:
	"""
	Test the firmware cache on the MIB controller, feature 7
	"""

	def _random_hex12(self, tmp, name, size):
		hexf = IntelHex16bit()

		for i in xrange(0, size):
			hexf[i] = random.randint(0, 0x3FFF)

		path = os.path.join(str(tmp), name)
		hexf.write_hex_file(path)

		return path

	def _fill_bucket(self, name, tmpdir, controller):
		modtype = random.randint(0,3)
		
		hexp = self._random_hex12(tmpdir, name, 4096)

		bucket = controller.push_firmware(hexp, modtype, verbose=False)
		readhex = controller.pull_firmware(bucket, verbose=False, pic12=True)

		outpath = os.path.join(str(tmpdir), 'output.hex')
		readhex.write_hex_file(outpath)

		return hexp, outpath

	def test_clear(self, controller):
		controller.clear_firmware_cache()

		res = controller.get_firmware_count()
		assert res['module_buckets'] == 0
		assert res['backup_firmware'] == False
		assert res['controller_firmware'] == False

	@pytest.mark.skipif(True, reason="Test is very slow")
	def test_buckets(self, controller, tmpdir):
		controller.clear_firmware_cache()

		in0, out0 = self._fill_bucket('hex0', tmpdir, controller)
		in1, out1 = self._fill_bucket('hex1', tmpdir, controller)
		in2, out2 = self._fill_bucket('hex2', tmpdir, controller)
		in3, out3 = self._fill_bucket('hex3', tmpdir, controller)

		res = controller.get_firmware_count()
		assert res['module_buckets'] == 4

	def test_info_empty(self, controller):
		controller.clear_firmware_cache()

		with pytest.raises(RPCException):
			controller.get_firmware_info(0)




from logstatement import LogStatement

class LogFinished (LogStatement):
	def __init__(self, statement, log):
		if len(statement['data']) != 0:
			raise ValueError('Invalid finished logging statement, data length != 0 (len=%d)' % len(statement['data']))

		self.address = log.current_address

	def format(self):
		return "Test Exited Successfully"


#!/usr/bin/env python

import sys
import os.path
import os
import intelhex
from time import sleep
import datetime

from pymomo.commander.meta import *
from pymomo.commander.exceptions import *
from pymomo.hex16.convert import *
from pymomo.utilities.typedargs import type_system
import cmdln
from colorama import Fore, Style
import pytest
from tempfile import NamedTemporaryFile

import struct

class ModTool(cmdln.Cmdln):
	name = 'modtool'

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_list(self, subcmd, opts):
		"""${cmd_name}: List all attached MIB modules on the bus
		
		${cmd_usage}
		${cmd_option_list}
		"""

		con = self._get_controller(opts)
		try:
			mods = con.enumerate_modules()
		except RPCException as e:
			print e.type
			print e.data

		print "Listing attached modules"
		for i, mod in enumerate(mods):
			print "%d: %s at address %d" % (i, mod.name, mod.address)

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-v', '--voltage', action='store_true', help='Display the MoMo battery voltage')
	def do_status(self, subcmd, opts):
		"""${cmd_name}: Report the attached momo device's status

		${cmd_usage}
		${cmd_option_list}
		"""

		con = self._get_controller(opts)

		if opts.voltage:
			print "Battery Voltage: %.2fV" % con.battery_status()

	def do_test(self, subcmd, opts, *tests):
		"""${cmd_name}: Run hardware tests on attached MoMo device

		Call with list as the only argument to see all of the different
		tests that are available.  Call with no arguments to run all tests.
		Call with test class names to run only those tests.

		${cmd_usage}
		${cmd_option_list}
		"""

		if len(tests) == 1 and tests[0] == 'list':
			test_string = '--collect-only'
		else:
			test_string = " ".join(map(lambda x: "-k " + x, tests))

		pytest.main('--pyargs pymomo %s' % test_string)

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_reset(self, subcmd, opts):
		con = self._get_controller(opts)
		con.reset()

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_factory_reset(self, subcmd, opts):
		really = raw_input("Are you sure you want to reset the device to factory defaults?  Data will be lost.  [y/N]: ")
		if ( really == "y" or really == "Y" or really == "yes" or really == "Yes" ):
			con = self._get_controller(opts)
			con.factory_reset()
		else:
			print "Cancelled!"

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_recover(self, subcmd, opts):
		con = self._get_controller(opts)

		print 'Attempting Recovery...'
		con.set_alarm(True)

		if not con.alarm_asserted():
			print "Could not set ALARM pin, bailing..."
			return 1

		raw_input("Please cycle power on the attached MoMo...(press return when done)")
		sleep(0.5)
		con.set_alarm(False)


		if not con.alarm_asserted():
			print "DID NOT DETECT REFLASH IN PROGRESS, AN ERROR PROBABLY OCCURRED..."
			return 1
		else:
			print 'Waiting for reflash to complete'

		while con.alarm_asserted():
			sys.stdout.write('.')
			sys.stdout.flush()
			sleep(0.1)

		print '\nReflash complete'

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_fsu(self, subcmd, opts, command):
		"""${cmd_name}: Directly control the FSU that is connected to the MoMo unit.  

		Possible subcommands are heartbeat, reset and attached.  
		- heartbeat checks if the FSU is still there.
		- reset resets the FSU.
		- attached determines if a MoMo unit is plugged in to the FSU.

		${cmd_usage}
		${cmd_option_list}
		"""

		commands = set(["reset", "heartbeat", "attached"])

		con = self._get_controller(opts)

		if command not in commands:
			print "Usage: modtool fsu [reset|heartbeat|attached]"
			return 1

		if command == "heartbeat":
			if con.stream.heartbeat() is False:
				print "FSU Heartbeat NOT DETECTED, try resetting it with modtool reset -f"
				return 1
			else:
				print "FSU Heartbeat detected"
		elif command == "reset":
			status = con.stream.reset()
			if status is False:
				print "FSU reset NOT SUCCESSFUL, try unplugging and replugging it from both the MoMo and computer at the same time."
				return 1
			else:
				print "FSU reset successful"
		elif command == "attached":
			if con.momo_attached():
				print "MoMo unit: Attached"
			else:
				print "NO MOMO UNIT ATTACHED"

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_alarm(self, subcmd, opts, *asserted):
		con = self._get_controller(opts)


		if len(asserted) == 0:
			status = con.alarm_asserted()

			if status:
				message = "Asserted"
			else:
				message = "Idle"
			print "Alarm status: %s" % message
		elif len(asserted) == 1:
			cmd = asserted[0]
			if cmd == "assert":
				con.set_alarm(True)
			elif cmd == "clear":
				con.set_alarm(False)
			else:
				print "Unknown command passed to modtool alarm: %s" % cmd

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-t', '--type', choices=['module', 'controller', 'backup'], default='module', help='What type of firmware module (module, controller, backup)')
	@cmdln.option('-c', '--clear', action='store_true', default=False, help='Clear the firmware cache before pushing')
	def do_push(self, subcmd, opts, hexfile):
		"""${cmd_name}: Push a firmware file to the attached momo unit.  

		You can either push the firmware into the 4 module firmware bins,
		the main controller firmware bin or the backup controller firmware
		bin.  Use -c to clear the controller's firmware cache before pushing.

		${cmd_usage}
		${cmd_option_list}
		"""

		con = self._get_controller(opts)

		if opts.clear:
			con.clear_firmware_cache()

		processed = hexfile

		if opts.type == "module":
			type = 0
			print "Pushing module firmware"
		elif opts.type == "controller":
			type = 5
			processed = self._convert_hex24(hexfile)
			print "Pushing (processed) main controller firmware"
		elif opts.type == "backup":
			type = 6
			processed = self._convert_hex24(hexfile)
			print "Pushing (processed) backup controller firmware"

		con.push_firmware(processed, type, verbose=True)

		#If we created a new processed file, remove it
		if processed != hexfile:
			os.remove(processed)

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-b', '--bucket', type=int, help='Firmware bucket to pull from (0-5)')
	def do_pull(self, subcmd, opts, hexfile):
		"""${cmd_name}: Pull a firmware file from the attached momo unit.  

		The firmware from the given bucket is pulled and saved to a file. 
		If the bucket is 4 or 5, the firmware is padded out to 4-byte words
		from 24 bit words since those two buckets are pic24 firmwares.  Otherwise
		the firmware is saved as is.

		${cmd_usage}
		${cmd_option_list}
		"""

		if opts.bucket >= 6:
			print "Invalid bucket %d: must be in [0, 5]" % opts.bucket

		if opts.bucket >= 4:
			pic12 = False
		else:
			pic12 = True

		con = self._get_controller(opts)
		hexobj = con.pull_firmware(opts.bucket, pic12=pic12, verbose=True)

		if not pic12:
			hexobj = pad_pic24_hex(hexobj)

		hexobj.write_hex_file(hexfile)


	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-l', '--length', choices=['8', '16', '24', '32'], default=16)
	def do_read(self, subcmd, opts, address):
		"""${cmd_name}: Read firmware directly from flash.

		${cmd_usage}
		${cmd_option_list}
		"""

		con = self._get_controller(opts)

		ltable = {'8': 1, '16': 2, '24': 3, '32': 4}

		addr = int(address, 0)
		res = con.read_flash(addr, ltable[opts.length], verbose=False)

		if opts.length == '8':
			num = res[0]
		elif opts.length == '16':
			num = res[0] | (res[1] << 8)
		elif opts.length == '24':
			num = res[0] | (res[1] << 8) | (res[2] << 16)
		elif opts.length == '32':
			num = res[0] | (res[1] << 8) | (res[2] << 16) | (res[3] << 24)

		print '0x%X' % num
		
	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_info(self, subcmd, opts, index):
		"""${cmd_name}: Describe the mib module given its index

		Index is returned by mobtool list
		${cmd_usage}
		${cmd_option_list}
		"""
		
		con = self._get_controller(opts)
		mod = con.describe_module(int(index))

		print "Module at index %d" % int(index)
		print "Name: %s" % mod.name
		print "Type: %d" % mod.type
		print "Flags: %d" % mod.flags
		print "Address: %d" % mod.address
		print "Features: %d" % mod.num_features

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', default=None, help='Select device by address')
	@cmdln.option('-n', '--name', default=None, help='Select device by name')
	@cmdln.option('-c', '--controller', action='store_true', default=False, help='Reflash the controller')
	@cmdln.option('--noreset', action='store_true', default=False, help='Do not reset the bus after the reflash')
	def do_reflash(self, subcmd, opts, hexfile):
		"""${cmd_name}: Reflash the mib12 module given either its name or address 

		If name is passed, the first mib module with that name is reflashed.  If
		address is passed, the address must match a currently attached module and
		that module will be reflashed.  You must pass either an address or a name,
		but not both.

		HEXFILE should be a valid mib12 application module.  The controller firmware
		cache will be cleared during the reflashing process and the module in question
		will be reset. 
		
		${cmd_usage}
		${cmd_option_list}
		"""

		con = self._get_controller(opts)

		if opts.controller:
			reflash_controller(con, hexfile)
		else:
			reflash_module(con, hexfile, name=opts.name, address=int(opts.address), noreset=opts.noreset)

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_time(self, subcmd, opts):
		"""${cmd_name}: Get the current RTCC time according to the controller module.

		${cmd_usage}
		${cmd_option_list}
		"""

		con = self._get_controller(opts)

		print con.current_time()

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_battery(self, subcmd, opts):
		"""${cmd_name}: Get the current battery voltage

		${cmd_usage}
		${cmd_option_list}
		"""

		con = self._get_controller(opts)

		print con.battery_status()

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_scheduler(self, subcmd, opts, command, index = None):
		"""${cmd_name}: Manage scheduled callbacks

		Possible subcommands are heartbeat, reset and attached.  
		- new creates a new dummy scheduled task (address 43, feature 20, command 8, frequency 1s)
		- remove removes the dummy scheduled task
		- map returns the map of task buckets
		- describe <index> describes the callback at index <index>

		${cmd_usage}
		${cmd_option_list}
		"""

		con = self._get_controller(opts)
		if command == "map":
			print bin( con.scheduler_map() )
		elif command == "new":
			con.scheduler_new(43, 20, 8, 1)
		elif command == "remove":
			con.scheduler_remove(43, 20, 8, 1)
		elif command == "describe":
			if index == None:
				print "You must specify a scheduler index to describe."
				exit(1)
			callback = con.scheduler_describe(index)
			if callback == None:
				print "No scheduled callback found at index %d" % int(index)
				exit(1)
			print "Address: %d" % int(callback[0])
			print "Feature: %d" % int(callback[1])
			print "Command: %d" % int(callback[2])
			if callback[3] == 0:
				frequency = "Every half second"
			elif callback[3] == 1:
				frequency = "Every second"
			elif callback[3] == 2:
				frequency = "Every 10 seconds"
			elif callback[3] == 3:
				frequency = "Every minute"
			elif callback[3] == 4:
				frequency = "Every 10 minutes"
			elif callback[3] == 5:
				frequency = "Every hour"
			elif callback[3] == 6:
				frequency = "Every day"
			else:
				frequency = "<unknown>"

			print "Frequency: %s" % frequency

		else:
			print "Invalid subcommand specified for command 'scheduler'."
			exit(1)

	def _get_controller(self, opts):
		try:
			c = get_controller(opts.port)
		except NoSerialConnectionException as e:
			print "Available serial ports:"
			if not e.available_ports():
				print "<none>"
			else:
				for port, desc, hwid in e.available_ports():
					print "\t%s (%s, %s)" % ( port, desc, hwid )
			self.error(str(e))
		except ValueError as e:
			self.error(str(e))

		return c

	def _convert_hex24(self, hexfile):
		tmpf = NamedTemporaryFile(delete=False)
		tmpf.close()

		tmp = tmpf.name

		out = unpad_pic24_hex(hexfile)
		out.write_hex_file(tmp)
		return tmp

	def error(self, text):
		print Fore.RED + "Error Occurred: " + Style.RESET_ALL + text
		sys.exit(1)

def main():
	type_system.interactive = True
	modtool = ModTool()
	return modtool.main()


from logstatement import LogStatement

class UnknownStatement (LogStatement):
	def __init__(self, statement, log):
		self.statement = statement
		self.pull_info(log)

	def keep(self):
		return True

	def format(self):
		return "Unknown Logging Statement (type=%d)" % self.statement['control']['value']


from pymomo.pcb.bom_matcher import BOMMatcher

class CacheOnlyMatcher (BOMMatcher):
	"""
	Only return matches from the existing cache.

	If a component is not in the cache, do not attempt to match
	it in any way.  This is useful for offline purposes and writing
	unit tests.
	"""


	def __init__(self):
		super(CacheOnlyMatcher, self).__init__()

	def _match(self):
		pass



#pic12_unit.py
#Routines for building unit tests for the pic12 processor
from SCons.Script import *
from SCons.Environment import Environment
import os
import os.path
import utilities
import pic12

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
from pymomo.gpysim import log
from pymomo.hex8 import symbols
from pymomo.exceptions import *

def build_unittest(test, arch, summary_env, cmds=None):
	"""
	Build a hex file from the source files test_files for the indicated chip
	If cmds is passed, replace the generic run command to gpsim with the commands
	listed in the passed file
	"""

	#Extract information from test
	#Allow files to be renamed with architecture extensions automatically
	test_files = filter(lambda x: x is not None, map(lambda x: test.find_support_file(x, arch), test.files))

	name = test.name
	type = test.type

	env = Environment(tools=['xc8_compiler', 'patch_mib12', 'merge_mib12_app', 'merge_mib12_sym', 'gpsim_runner'], ENV = os.environ)
	env['ORIGINAL_ARCH'] = arch
	env['TEST'] = test

	#Configure for app module or exec
	if type.startswith("executive"):
		orig_name = 'mib12_executive_symbols'
		env['ARCH'] = arch.retarget(remove=['exec'], add=['app'])
		pic12.configure_env_for_xc8(env, force_app=True)
		test_harness = ['../test/pic12/exec_harness/mib12_exec_unittest.c', '../test/pic12/exec_harness/mib12_api.as', '../test/pic12/exec_harness/mib12_exec_unittest_startup.as', '../test/pic12/gpsim_logging/test_log.as', '../test/pic12/gpsim_logging/test_mib.as']
	elif type == "application":
		orig_name = "mib12_app_module_symbols"

		env['ARCH'] = arch.retarget(remove=['app'], add=['exec'])

		pic12.configure_env_for_xc8(env, force_exec=True)
		test_harness = ['../test/pic12/app_harness/mib12_app_unittest.c', '../test/pic12/app_harness/mib12_test_api.as', '../test/pic12/gpsim_logging/test_log.as']
	else:
		raise BuildError("Invalid unit test type specified. Should start with executive or application.", type=type)

	orig_symfile = orig_name + '.h'
	orig_symtab = orig_name + '.stb'

	dirs = arch.build_dirs()

	builddir = dirs['build']
	testdir = os.path.join(dirs['test'], name, 'objects')
	finaldir = dirs['output']
	outdir = os.path.join(dirs['test'], name)
	
	env.AppendENVPath('PATH','../../tools/scripts')

	incs = []
	incs.append('.')
	incs.append('src')
	incs.append('src/mib')
	incs.append(testdir)
	incs.extend(arch.property('test_includes', []))

	env['INCLUDE'] += incs

	#Copy over the symbol file from the module we're testing so we can reference it
	symfile = Command(os.path.join(testdir, 'symbols.h'),  os.path.join(builddir, orig_symfile), Copy("$TARGET", "$SOURCE"))
	testee_symtab = Command(os.path.join(testdir, 'symbols.stb'),  os.path.join(builddir, orig_symtab), Copy("$TARGET", "$SOURCE"))

	symtab = env.merge_mib12_symbols([os.path.join(outdir, 'symbols.stb')], [testee_symtab, os.path.join(testdir, name + '_unit.sym')])

	#Load in all of the xc8 configuration from build_settings
	sim = arch.property('gpsim_proc')

	env['TESTCHIP'] = sim
	env['TESTNAME'] = name
	env['TESTAPPEND'] = arch.arch_name()
	env['EXTRACMDS'] = cmds

	#Must do this in 1 statement so we don't modify test_files
	srcfiles = test_files + test_harness

	apphex = env.xc8(os.path.join(testdir, name + '_unit.hex'), srcfiles)
	env.Depends(apphex[0], symfile)

	if type.startswith("executive"):
		app_start = env['CHIP'].app_rom[0] + 2

		#Executive integration tests run the entire executive startup routine
		#Regular integration tests skip it to save time (1 second delay for registration)
		#and to avoid triggering any bugs in the executive code since these are unit tests.
		#for specific routines.
		if type == "executive_integration":
			lowhex = os.path.join(builddir, 'mib12_executive_patched.hex')
		else:
			lowhex = env.Command(os.path.join(testdir, 'mib12_executive_local.hex'), os.path.join(builddir, 'mib12_executive_patched.hex'), action='python ../../tools/scripts/patch_start.py %d $SOURCE $TARGET' % app_start)
		highhex = apphex[0]
	else:
		lowhex = apphex[0]
		highhex = env.Command(os.path.join(testdir, 'mib12_app_module_local.hex'), os.path.join(builddir, 'mib12_app_module.hex'), Copy("$TARGET", "$SOURCE"))
	
	outhex = env.merge_mib12_app(os.path.join(outdir, name + '.hex'), [lowhex, highhex])

	outscript = env.Command([os.path.join(outdir, 'test.stc')], [outhex], action=env.Action(build_unittest_script, "Building test script"))

	raw_log_path = os.path.join(outdir, build_logfile_name(env))

	raw_results = env.gpsim_run(raw_log_path, [outscript, outhex]) #include outhex so that scons knows to rerun this command when the hex changes
	formatted_log = env.Command([build_formatted_log_name(env), build_status_name(env)], [raw_results, symtab], action=env.Action(process_unittest_log, 
		"Processing test log"))

	#Add this unit test to the unit test summary command
	summary_env['TESTS'].append(build_status_name(env))

	#Remember to remove the test directory when cleaning
	#Also add any extra intermediate files that the unit test defines so that 
	#those are cleaned up as well
	env.Clean(outscript, testdir)
	env.Clean(outscript, outdir)
	additional_files = test.get_intermediates(arch)
	for file in additional_files:
		env.Clean(outscript, file)

def build_exec_unittest(test_files, name, chip):
	"""
	Build the unit test described by the source files <test_files>.  Build the 
	MIB12 executive hex file and then a hex file containing the unit test,
	join them together and build gpsim script to execute the test.  Create a combined
	symbol file for the executive and test so that the log file generated by the test
	can be interpreted and addressed mapped to functions.
	"""

	build_unittest(test_files, name, chip, type="executive")

def build_unittest_script(target, source, env):
	"""
	Build a gpsim script to execute this unit test
	"""

	logfile = env['TESTNAME'] + '@' + env['TESTAPPEND'] + '.raw'

	sim = env['TESTCHIP']
	name = env['TESTNAME']

	extracmds = None

	if 'EXTRACMDS' in env and env['EXTRACMDS'] is not None:
		with open(env['EXTRACMDS'], "r") as f:
			extracmds = f.readlines()

	with open(str(target[0]), "w") as f:
		f.write('processor %s\n' % sim)
		f.write('load s %s\n' % os.path.basename(str(source[0])))
		f.write('break w 0x291, reg(0x291) == 0x00\n')
		f.write('log w %s.ccpr1l\n' % sim)
		f.write('log w %s.ccpr1h\n' % sim)
		f.write("log on '%s'\n" % logfile)
		if extracmds is not None:
			for cmd in extracmds:
				f.write(cmd)
		else:
			f.write('run\n')

		f.write('quit\n')

def process_unittest_log(target, source, env):
	"""
	Source should be the unprocessed log file and the symbol file (stb) for assigning addresses to functions.
	"""
	test = env['TEST']
	symtab = symbols.XC8SymbolTable(str(source[1]))
	lf = log.LogFile(str(source[0]), symtab=symtab)
	lf.save(str(target[0]))

	passed = lf.test_passed(test)
	if passed:
		passed = test.check_output(env['ORIGINAL_ARCH'], str(target[0]))

	test.save_status(str(target[1]), passed)

def build_summary_name():
	return os.path.join('build', 'test', 'output', 'results.txt')

def build_logfile_name(env):
	return env['TESTNAME'] + '@' + env['TESTAPPEND'] + '.raw'

def build_formatted_log_name(env):
	return os.path.join('build', 'test', 'output', 'logs', env['TESTNAME'] + '@' + env['TESTAPPEND'] + '.log')

def build_status_name(env):
	return os.path.join('build', 'test', 'output', 'logs', env['TESTNAME'] + '@' + env['TESTAPPEND'] + '.status')




import serial
import transport
import io

class SerialTransport (transport.Transport):
	"""
	Standard serial port transport layer
	"""

	def __init__(self, dev, baud=115200, timeout=120):
		self.io = serial.Serial(port=dev, baudrate=baud, timeout=timeout)

	def write(self, buffer):
		for b in buffer:
			self.io.write(b)
		
		self.io.flush()

	def read(self, cnt=1):
		buf = self.io.read(cnt)
		return buf

	def close(self):
		self.port.close

	def receive_count(self):
		return self.io.inWaiting()


import os.path
import unittest
from pymomo.sim import simulator
from pymomo.utilities.config import ConfigFile
from distutils.spawn import find_executable

@unittest.skipIf(find_executable("sim30") == None, 'SIM30 executable not found')
class TestSIM30BasicFunctionality(unittest.TestCase):
	"""
	Test to make sure the SIM30 pic24 simulator is working
	correctly
	"""
	def setUp(self):
		self.sim = simulator.Simulator('pic24')

	def tearDown(self):
		self.sim.exit()

	def test_basic_functionality(self):
		self.assertTrue(self.sim.ready())


from pymomo.commander import transport, cmdstream
import pymomo.commander.proxy
from pymomo.commander.exceptions import *
from pymomo.utilities.typedargs.annotate import param

import serial
from serial.tools import list_ports
import re

def get_controller(serial_port=None):
	"""
	Given serial port descriptor, create all of the necessary
	object to get a controller proxy module 
	"""

	if serial_port == None:
		serial_port = _find_momo_serial()
		if serial_port == None:
			raise NoSerialConnectionException( _get_serial_ports() )

	s = transport.SerialTransport(serial_port)
	c = cmdstream.CMDStream(s)

	con = pymomo.commander.proxy.MIBController(c)
	return con

def _get_serial_ports():
	return list_ports.comports()

def _find_momo_serial():
	"""
	Iterate over all connected COM devices and return the first
	one that matches FTDI's Vendor ID (403)
	"""

	for port, desc, hwid in _get_serial_ports():
		if (re.match( r"USB VID:PID=0?403:6015", hwid) != None) or (re.match( r".*VID_0?403.PID_6015", hwid) != None):
			return port

#Annotated functions for use with momo tool
@param('serial', 'string', desc='the serial port to use')
def controller(serial=""):
	"""
	Given serial port descriptor, create all of the necessary
	object to get a controller proxy module 
	"""

	if serial == "":
		serial = None

	return get_controller(serial)


#!/usr/bin/env python

import sys, os

from pymomo.commander.meta import *
from pymomo.commander.exceptions import *
from pymomo.commander.proxy import *

import cmdln

class GSMTool(cmdln.Cmdln):
	name = 'gsmtool'

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the GSM module' )
	def do_on(self, subcmd, opts):
		"""${cmd_name}: Turn on the GSM modem.

		${cmd_usage}
		${cmd_option_list}
		"""
		gsm = self._create_proxy( opts )
		if gsm.module_on(): 
			print "OK"
		else:
			print "ERROR"

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the GSM module' )
	def do_off(self, subcmd, opts):
		"""${cmd_name}: Turn off the GSM modem

		${cmd_usage}
		${cmd_option_list}
		"""
		gsm = self._create_proxy( opts )
		gsm.module_off()

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the GSM module' )
	def do_dump(self, subcmd, opts):
		"""${cmd_name}: Dump the modem's serial buffer (for debugging)

		${cmd_usage}
		${cmd_option_list}
		"""
		gsm = self._create_proxy( opts )
		res = gsm.dump_buffer()
		print "(%d): %s" % (len(res), res)

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the GSM module' )
	def do_debug(self, subcmd, opts):
		"""${cmd_name}: Get debug info

		${cmd_usage}
		${cmd_option_list}
		"""
		gsm = self._create_proxy( opts )
		res = gsm.debug()
		print "module_on: %s" % bool(ord(res[0]))
		print "callback_pending: %s" % bool(ord(res[1]))
		print "rx_buffer_start: %d" % ord(res[2])
		print "rx_buffer_end: %d" % ord(res[3])
		print "rx_buffer_len: %d" % ord(res[4])
		print "debug_val: %d" % ord(res[5])
		print "http_status: %d" % ( ord(res[6]) | ord(res[7])<<8)

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the GSM module' )
	def do_cmd(self, subcmd, opts, cmd):
		"""${cmd_name}: Tell the GSM modem to execute an AT command

		${cmd_usage}
		${cmd_option_list}
		"""
		gsm = self._create_proxy( opts )
		res = gsm.at_cmd( cmd )
		print res

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the GSM module' )
	def do_msg(self, subcmd, opts, dest, text ):
		"""${cmd_name}: Send a text message with the content <text> to <dest>

		${cmd_usage}
		${cmd_option_list}
		"""
		gsm = self._create_proxy( opts )
		res = gsm.send_message( dest, text );

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the GSM module' )
	def do_apn(self, subcmd, opts, apn ):
		"""${cmd_name}: Set the GPRS APN

		${cmd_usage}
		${cmd_option_list}
		"""
		gsm = self._create_proxy( opts )
		gsm.set_apn(apn);
		print "OK"

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the GSM module' )
	def do_gprs(self, subcmd, opts ):
		"""${cmd_name}: Test GPRS functionality

		${cmd_usage}
		${cmd_option_list}
		"""
		gsm = self._create_proxy( opts )
		res = gsm.test_gprs();
		print res

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the GSM module' )
	def do_simtest(self, subcmd, opts ):
		"""${cmd_name}: Test that a SIM card is inserted

		${cmd_usage}
		${cmd_option_list}
		"""
		gsm = self._create_proxy( opts )
		res = gsm.rpc(10,3,result_type=(1, False));
		print res['ints'][0] != 0

	def _create_proxy(self,opts):
		try:
			con = get_controller(opts.port)
			address = 11
			if opts.address != None:
				address = opts.address
			gsm = GSMModule(con.stream, address)
		except NoSerialConnectionException as e:
			print "Available serial ports:"
			if not e.available_ports():
				print "<none>"
			else:
				for port, desc, hwid in e.available_ports():
					print "\t%s (%s, %s)" % ( port, desc, hwid )
			self.error(str(e))
		except ValueError as e:
			self.error(str(e))

		return gsm

def main():
	gsmtool = GSMTool()
	return gsmtool.main()


#!/usr/bin/env python

import sys, os

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from pymomo.commander.meta import *
from pymomo.commander.exceptions import *
from pymomo.commander.proxy import *

import cmdln
import struct
import base64
import datetime

from random import randint

def chunk(l, n):
    for i in xrange(0, len(l), n):
        yield l[i:i+n]

class Reportinator(cmdln.Cmdln):
	name = 'reportinator'

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_reset(self, subcmd, opts):
		"""${cmd_name}: Reset report configuration

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller( opts )
		con.reset_reporting_config()

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_start(self, subcmd, opts):
		"""${cmd_name}: Start reporting

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller( opts )
		con.start_reporting()

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_stop(self, subcmd, opts):
		"""${cmd_name}: Stop reporting

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller( opts )
		con.stop_reporting()

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_state(self, subcmd, opts):
		"""${cmd_name}: Get the current state of the autonomous reporting flag.

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller( opts )
		if con.get_reporting():
			print "Enabled"
		else:
			print "Disabled"


	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_send(self, subcmd, opts):
		"""${cmd_name}: Send a single report

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller( opts )
		con.send_report();

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_routes(self, subcmd, opts, primary=None, secondary=None):
		"""${cmd_name}: Get or set the destination address for reporting.

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller( opts )
		if primary != None:
			con.set_report_route(0, primary)
			if secondary == "" or secondary != None:
				con.set_report_route(1, secondary)

		primary = con.get_report_route(0);
		secondary = con.get_report_route(1);
		print "primary:   %s" % (primary)
		print "secondary: %s" % (secondary)

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_apn(self, subcmd, opts, apn=None):
		"""${cmd_name}: Get or set the GPRS APN.

		${cmd_usage}
		${cmd_option_list}
		"""

		con = self._get_controller( opts )
		if apn != None:
			con.set_gprs_apn(apn)
		print con.get_gprs_apn()

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_interval(self, subcmd, opts, interval=None):
		"""${cmd_name}: Get or set the reporting interval.

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller( opts )
		if ( interval != None ):
			con.rpc( 60, 0x03, int(interval))
		interval = con.rpc( 60, 0x04, result_type=(1,False) );
		print "%d" % interval['ints'][0]

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_flags(self, subcmd, opts, flags=None):
		"""${cmd_name}: Get or set the report flags.

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller( opts )
		if ( flags != None ):
			con.rpc( 60, 0x07, int(flags))
		flags = con.rpc( 60, 0x08, result_type=(1,False) );
		print "%d" % flags['ints'][0]

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_aggregates(self, subcmd, opts, bulk=None, interval=None):
		"""${cmd_name}: Get or set the report aggregates.

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller( opts )
		if ( bulk != None and interval != None ):
			con.rpc( 60, 0x09, int(bulk), int(interval))
		aggregates = con.rpc( 60, 0x0A, result_type=(2,False) );
		print "bulk:     %d" % aggregates['ints'][0]
		print "interval: %d" % aggregates['ints'][1]

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_sequence(self, subcmd, opts):
		"""${cmd_name}: Get the current report sequence.

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller( opts )
		sequence = con.rpc( 60, 0x0B, result_type=(1,False) );
		print "%d" % sequence['ints'][0]

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_log(self,subcmd, opts, cmd, limit=None):
		"""${cmd_name}: Manage the log of reports, succeeded and failed.

		Subcommands:
		- count - return the number of reports in the log
		- read <limit> - read <limit> (or all if <limit> is omitted) reports from the log
		- clear - clear the log (USE AT YOUR OWN RISK)

		${cmd_usage}
		${cmd_option_list}
		"""

		con = self._get_controller(opts)
		if cmd == 'read':
			index = 0
			if limit != None:
				count = con.rpc( 60, 0x10, result_type=(1,False) )['ints'][0]
				limit = int(limit)
				if limit > count:
					limit = count
				index = count - limit
			while True:
				offset = 0
				report = ''
				while offset < 118:
					try:
						res = con.rpc( 60, 0x0F, index, offset, result_type=(0,True) )
					except RPCException as e:
						if e.type != 7:
							raise e
						break
					offset += len(res['buffer'])
					report += res['buffer']
				if len(report) == 0:
					break
				print base64.b64encode(report)
				index += 1
		elif cmd == 'count':
			res = con.rpc( 60, 0x10, result_type=(1,False) )
			print res['ints'][0]
		elif cmd == 'clear':
			res = con.rpc( 60, 0x11 )
			print "All items cleared from the report log."
		else:
			print "Invalid subcommand passed to log command."

	def do_parse(self, subcmd, opts, report):
		"""${cmd_name}: Parse a report in BASE64 format

		${cmd_usage}
		${cmd_option_list}
		"""

		agg_names = ['count','sum','mean','<none>','<none>','min','max']
		interval_types = ['second', 'minute', 'hour', 'day']
		report = base64.b64decode(report)

		version = ord(report[0])
		if version == 2:
			sensor, sequence, flags, battery_voltage, diag1, diag2, bulk_aggs, int_aggs, interval_def, interval_count = struct.unpack( "xBHHHHHBBBB", report[:16] )

			interval_type = interval_def & 0xF
			interval_step = interval_def >> 4
			if interval_type > len(interval_types):
				interval_type = "<unknown:%d>" % interval_type
			else:
				interval_type = interval_types[interval_type]

			bulk_agg_names = []
			for i in range(0,7):
				if ( bulk_aggs & (0x1 << i) ):
					bulk_agg_names.append(agg_names[i])

			int_agg_names = []
			for i in range(0,7):
				if ( int_aggs & (0x1 << i) ):
					int_agg_names.append(agg_names[i])

			print "version: 2"
			print "sensor: %d" % sensor
			print "sequence: %d" % sequence
			print "flags: %d" % flags
			print "battery charge: %.2fV" % ( float(battery_voltage) / 1024 * 2.78 * 2)
			print "diag: %d, %d" % (diag1, diag2)
			print "Interval type: %s" % interval_type
			print "         step: %d" % interval_step
			print "        count: %d" % interval_count

			if bulk_aggs != 0:
				print "Global aggregates:"
				values = struct.unpack( "H" * len(bulk_agg_names), report[16:16+len(bulk_agg_names)*2] )
				for i in range( len(values) ):
					print "\t%s = %d" % (bulk_agg_names[i], values[i])

			if int_aggs != 0:
				print "Interval Aggregates:"
				interval_aggregate_slice = report[16+len(bulk_agg_names)*2:16+len(bulk_agg_names)*2+interval_count*len(int_agg_names)*2]
				values = struct.unpack( "H" * len(int_agg_names) * interval_count, interval_aggregate_slice )
				row_format ="{:>8}" * (len(int_agg_names) + 1)
				print row_format.format("", *int_agg_names)
				for row in chunk(values, len(int_agg_names)):
					print row_format.format("", *row)
		elif version == 3:
			sequence, uuid, flags, timestamp, battery_voltage, bulk_aggs, int_aggs, interval_def, interval_count = struct.unpack( "<xBLHLHBBBB", report[:18] )

			interval_type = interval_def & 0xF
			interval_step = interval_def >> 4
			if interval_type > len(interval_types):
				interval_type = "<unknown:%d>" % interval_type
			else:
				interval_type = interval_types[interval_type]

			bulk_agg_names = []
			for i in range(0,7):
				if ( bulk_aggs & (0x1 << i) ):
					bulk_agg_names.append(agg_names[i])

			int_agg_names = []
			for i in range(0,7):
				if ( int_aggs & (0x1 << i) ):
					int_agg_names.append(agg_names[i])

			print "version: 3"
			print "transmit sequence: %d" % sequence
			uuid = struct.pack('<L', uuid)
			print "UUID: %s" % base64.b64encode(uuid).rstrip('=')
			print "flags: %d" % flags
			print "timestamp: %s" % datetime.datetime.utcfromtimestamp(timestamp + 946684800)
			print "battery charge: %.2fV" % ( float(battery_voltage) / 1024 * 2.78 * 2)
			print "Interval type: %s" % interval_type
			print "         step: %d" % interval_step
			print "        count: %d" % interval_count

			if bulk_aggs != 0:
				print "Global aggregates:"
				values = struct.unpack( "H" * len(bulk_agg_names), report[18:18+len(bulk_agg_names)*2] )
				for i in range( len(values) ):
					print "\t%s = %d" % (bulk_agg_names[i], values[i])

			if int_aggs != 0:
				print "Interval Aggregates:"
				interval_aggregate_slice = report[18+len(bulk_agg_names)*2:18+len(bulk_agg_names)*2+interval_count*len(int_agg_names)*2]
				values = struct.unpack( "H" * len(int_agg_names) * interval_count, interval_aggregate_slice )
				row_format ="{:>8}" * (len(int_agg_names) + 1)
				print row_format.format("", *int_agg_names)
				for row in chunk(values, len(int_agg_names)):
					print row_format.format("", *row)
		else:
			print "Unrecognized report version %d" % ord(report[0])
			return

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_test(self, subcmd, opts):
		con = self._get_controller( opts )
		
		second = 0
		minute = 0
		hour = 0
		day = 1

		con.set_time( 14, 1, day, hour, minute, second )
		while ( True ):
			con.sensor_log( 0, 0, randint(1,1000) )
			
			second += 60
			if ( second == 60 ):
				second = 0
				minute += 1
				if ( minute == 60 ):
					minute = 0
					hour += 1
					if hour == 24:
						hour = 0
						day += 1
						break
			print day, hour, minute, second
			con.set_time( 14, 1, day, hour, minute, second )

	def _get_controller(self,opts):
		try:
			con = get_controller(opts.port)
		except NoSerialConnectionException as e:
			print "Available serial ports:"
			if not e.available_ports():
				print "<none>"
			else:
				for port, desc, hwid in e.available_ports():
					print "\t%s (%s, %s)" % ( port, desc, hwid )
			self.error(str(e))
		except ValueError as e:
			self.error(str(e))

		return con

def main():
	reportinator = Reportinator()
	return reportinator.main()


#typeinfo.py
#Basic routines for converting information from string or other binary
#formats to python types and for displaying those types in supported 
#formats
#TODO: 
#- Extend the type system to use a recursive parser to allow complex
#  types to be built from other complex types.

from pymomo.exceptions import *
import types
import os.path
import imp

#Start working on recursive parser
#import pyparsing
#symbolchars = pyparsing.Regex('[_a-zA-Z][_a-zA-Z0-9]*')
#typename = pyparsing.Word(symbolchars)

#simpletype = typename
#complextype = pyparsing.Forward()

#typelist = pyparsing.delimitedList(simpletype | complextype, ',')
#complextype << typename + pyparsing.Literal('(').suppress() + typelist + pyparsing.Literal(')').suppress()

#statement = 

class TypeSystem(object):
	"""
	TypeSystem permits the inspection of defined types and supports
	converted string and binary values to and from these types.
	"""

	def __init__(self, *args):
		"""
		Create a TypeSystem by importing all of the types defined in modules passed
		as arguments to this function.  Each module is imported using 
		"""

		self.interactive = False
		self.known_types = {}
		self.type_factories = {}

		for arg in args:
			self.load_type_module(arg)

	def convert_to_type(self, value, type, **kwargs):
		"""
		Convert value to type 'type'

		If the conversion routine takes various kwargs to 
		modify the conversion process, **kwargs is passed
		through to the underlying conversion function
		"""

		typeobj = self.get_type(type)

		conv = typeobj.convert(value, **kwargs)
		return conv

	def get_type_size(self, type):
		"""
		Get the size of this type for converting a hex string to the
		type. Return 0 if the size is not known.
		"""

		typeobj = self.get_type(type)

		if hasattr(typeobj, 'size'):
			return typeobj.size()

		return 0

	def format_value(self, value, type, format=None, **kwargs):
		"""
		Convert value to type and format it as a string

		type must be a known type in the type system and format,
		if given, must specify a valid formatting option for the
		specified type.
		"""

		typed_val = self.convert_to_type(value, type, **kwargs)
		typeobj = self.get_type(type)

		#Allow types to specify default formatting functions as 'default_formatter'
		#otherwise if not format is specified, just convert the value to a string
		if format is None:
			if hasattr(typeobj, 'default_formatter'):
				format_func = getattr(typeobj, 'default_formatter')
				return format_func(typed_val, **kwargs)

			return str(typed_val)

		formatter = "format_%s" % str(format)
		if not hasattr(typeobj, formatter):
			raise ArgumentError("Unknown format for type", type=type, format=format, formatter_function=formatter)

		format_func = getattr(typeobj, formatter)
		return format_func(typed_val, **kwargs)

	def _validate_type(self, typeobj):
		"""
		Validate that all required type methods are implemented.

		At minimum a type must have:
		- a convert() or convert_binary() function
		- a default_formatter() function

		Raises an ArgumentError if the type is not valid
		"""

		if not (hasattr(typeobj, "convert") or hasattr(typeobj, "convert_binary")):
			raise ArgumentError("type is invalid, does not have convert or convert_binary function", type=typeobj, methods=dir(typeobj))

		if not hasattr(typeobj, "default_formatter"):
			raise ArgumentError("type is invalid, does not have default_formatter function", type=typeobj, methods=dir(typeobj))

	def is_known_type(self, type):
		"""
		Check if type is known to the type system.

		Returns boolean indicating if type is known.
		"""

		if not isinstance(type, basestring):
			raise ArgumentError("type must be a string naming a known type", type=type)

		if type in self.known_types:
			return True

		return False

	def split_type(self, typename):
		"""
		Given a potentially complex type, split it into its base type and specializers
		"""

		name = self._canonicalize_type(typename)
		if '(' not in name:
			return name, False, []

		base,sub = name.split('(')
		if len(sub) == 0 or sub[-1] != ')':
			raise ArgumentError("syntax error in complex type, no matching ) found", passed_type=typename, basetype=base, subtype_string=sub)
		
		sub = sub[:-1]

		subs = sub.split(',')
		return base, True, subs

	def instantiate_type(self, typename, base, subtypes):
		"""
		Instantiate a complex type
		"""

		if base not in self.type_factories:
			raise ArgumentError("unknown complex base type specified", passed_type=typename, base_type=base)

		BaseType = self.type_factories[base]

		#Make sure all of the subtypes are valid
		for s in subtypes:
			try:
				self.get_type(s)
			except MoMoException as e:
				raise ArgumentError("could not instantiate subtype for complex type", passed_type=typename, sub_type=s, error=e)

		typeobj = BaseType.Build(*subtypes, type_system=self)
		self.inject_type(typename, typeobj)

	def _canonicalize_type(self, typename):
		return typename.replace(' ', '')

	def get_type(self, typename):
		"""
		Return the type object corresponding to a type name.
		"""

		typename = self._canonicalize_type(typename)

		type, is_complex, subtypes = self.split_type(typename)
		if not self.is_known_type(typename):
			if is_complex:
				self.instantiate_type(typename, type, subtypes)
			else:
				raise ArgumentError("get_type called on unknown type", type=typename)

		return self.known_types[typename]

	def is_known_format(self, type, format):
		"""
		Check if format is known for given type.

		Returns boolean indicating if format is valid for the specified type.
		"""

		typeobj = self.get_type(type)

		formatter = "format_%s" % str(format)
		if not hasattr(typeobj, formatter):
			return False

		return True

	def _is_factory(self, typeobj):
		"""
		Determine if typeobj is a factory for producing complex types
		"""

		if hasattr(typeobj, 'Build'):
			return True

		return False

	def format_return_value(self, function, value):
		"""
		Format the return value of a function based on the annotated type information
		"""

		return self.format_value(value, function.retval_typename, function.retval_formatter)

	def inject_type(self, name, typeobj):
		"""
		Given a module-like object that defines a type, add it to our type system so that
		it can be used with the momo tool and with other annotated API functions.
		"""

		name = self._canonicalize_type(name)
		base,is_complex,subs = self.split_type(name)

		if self.is_known_type(name):
			raise ArgumentError("attempting to inject a type that is already defined", type=name)

		if (not is_complex) and self._is_factory(typeobj):
			if name in self.type_factories:
				raise ArgumentError("attempted to inject a complex type factory that is already defined", type=name)
			self.type_factories[name] = typeobj
		else:
			self._validate_type(typeobj)
			self.known_types[name] = typeobj

		if not hasattr(typeobj, "default_formatter"):
			raise ArgumentError("type is invalid, does not have default_formatter function", type=typeobj, methods=dir(typeobj))

	def load_type_module(self, module, verbose=False):
		"""
		Given a module that contains a list of some types find all symbols in the module that 
		do not start with _ and attempt to import them as types.
		"""

		for name in filter(lambda x: not x.startswith('_'), dir(module)):
			typeobj = getattr(module, name)
			self.inject_type(name, typeobj)

	def load_external_types(self, path, verbose=False):
		"""
		Given a path to a python package or module, load that module, search for all defined variables
		inside of it that do not start with _ or __ and inject them into the type system.  If any of the
		types cannot be injected, silently ignore them unless verbose is True.  If path points to a module
		it should not contain the trailing .py since this is added automatically by the python import system
		"""

		d,p = os.path.split(path)

		try:
			fileobj,pathname,description = imp.find_module(p, [d])
			mod = imp.load_module(p, fileobj, pathname, description)
		except ImportError as e:
			raise ArgumentError("could not import module in order to load external types", module_path=path, parent_directory=p, module_name=p, error=str(e))

		self.load_type_module(mod, verbose)

		#TODO add checking for types that could not be injected and report them

#In order to support function annotations that must be resolved to types when modules
#are imported, create a default TypeSystem object that is used globally to store type
#information

type_system = TypeSystem(types)



from logstatement import LogStatement

class LogAssert (LogStatement):
	def __init__(self, statement, log):
		if len(statement['data']) != 2:
			raise ValueError('Invalid assert logging statement, data length != 2 (len=%d)' % len(statement['data']))

		self.actual = statement['data'][0]['value']
		self.desired = statement['data'][1]['value']

		self.address = log.current_address

	def format(self):
		return "ASSERT FAILED (was 0x%X, wanted 0x%X)" % (self.actual, self.desired)

	def error(self):
		return True


#MoMo Exceptions

class MoMoException(Exception):
	"""
	All MoMo API routines, upon encountering unrecoverable errors,
	will throw an exception that is a subclass of this exception.
	All other exceptions are caught and rethrown as an APIError
	for consistency.  Calling methods can assume that all annotated
	API functions will throw only MoMoException subclasses as defined
	below.
	"""

	def __init__(self, msg, **kwargs):
		self.params = kwargs
		self.msg = msg
	
	def format(self):
		msg = "%s: %s" % (self.__class__.__name__, self.msg)

		if len(self.params) != 0:
			paramstring = "\n".join([str(key) + ": " + str(val) for key,val in self.params.iteritems()])
			msg += "\nAdditional Information:\n" + paramstring
		
		return msg

	def format_msg(self):
		msg = self.msg
		if len(self.params) != 0:
			paramstring = "\n".join([str(key) + ": " + str(val) for key,val in self.params.iteritems()])
			msg += "\nAdditional Information:\n" + paramstring
		
		return msg

	def __str__(self):
		msg = self.format()
		return msg

class ValidationError(MoMoException):
	"""
	API routines can impose validation criteria on their arguments in 
	addition to requiring simply a certain type of argument.  A clasic
	example is the "path" type which can have validators like "readable"
	or "exists".  When validation fails, this error is thrown.
	"""

	pass

class ConversionError(MoMoException):
	"""
	All API functions take typed parameters.  Each type defines conversion 
	operators for python types that are logically related to it.  When no
	valid conversion exists for the data type passed, this error is thrown.
	"""

	pass

class NotFoundError(MoMoException):
	"""
	Thrown when an attempt to execute an API method is made and the API
	method does not exist.
	"""

	pass

class TimeoutError(MoMoException):
	"""
	The method timed out, usually indicating that a communication failure
	occurred, either with another process or with a hardware module.
	"""

	pass

class ArgumentError(MoMoException):
	"""
	The method could not be called with the arguments passed.  This
	differs from InternalError in that when ArgumentError is returned,
	it is known that these arguments can never work for this function.

	An example would be passing three arguments to a function requiring
	4 arguments.
	"""

	pass

class DataError(MoMoException):
	"""
	The method relied on data pass in by the user and the data was invalid.

	This could be because a file was the wrong type or because a data provider
	returned an unexpected result.  The parameters passed with this exception
	provide more detail on what occurred and where.
	"""

	pass

class InternalError(MoMoException):
	"""
	The method could not be completed with the user input passed for 
	an unexpected reason.  This does not signify a bug in the API 
	method code.  More details should be passed in the arguments.
	"""

	pass

class APIError(MoMoException):
	"""
	An internal API error occured during the execution of the method.
	This should only be returned if the error was unforeseen and not
	caused in any way by user input.  If the problem is that a user 
	input is invalid for the API call, ValidationError should be
	thrown instead.  

	All instances of APIError being thrown are bugs that should be
	reported and fixed. 
	"""

	pass

class BuildError(MoMoException):
	"""
	There is an error with some part of the build system.  This does not
	mean that there is a compilation error but rather that a required part
	of the build process did not complete successfully.  This exception means
	that something is misconfigured.
	"""

	pass

class TypeSystemError(MoMoException):
	"""
	There was an error with the MoMo type system.  This can be due to improperly
	specifying an unknown type or because the required type was not properly loaded
	from an external module before a function that used that type was needed.
	"""

	pass

class EnvironmentError(MoMoException):
	"""
	The environment is not properly configured for the MoMo API command that was called.
	This can be because a required program was not installed or accessible or because
	a required environment variable was not defined.
	"""

	pass


from pycparser import parse_file, c_ast
import utilities

class FunctionDefinitionRecorder(c_ast.NodeVisitor):
	def __init__(self):
		self.defined_functions = []
		self.def_locations = []

	def visit_FuncDef(self, node):
		self.defined_functions.append(node.decl.name)
		self.def_locations.append(node.decl.coord)

class ParsedCFile:
	"""
	An object allowing one to explore the AST of a C file.  The file is 
	parsed using pycparser and various convenience routines are given to
	speed access to certain parts of the file.  
	"""

	def __init__(self, filepath, arch):
		self.filepath = filepath
		self.arch = arch

		self._parse_file()

	def _parse_file(self):
		"""
		Preprocess and parse C file into an AST
		"""

		#We need to set the CPU type to pull in the right register definitions
		#only preprocess the file (-E) and get rid of gcc extensions that aren't
		#supported in ISO C.
		args = utilities.build_includes(self.arch.includes())
		args.append('-mcpu=%s' % self.arch.property('chip'))
		args.append('-E')
		args.append('-D__attribute__(x)=')
		args.append('-D__extension__=')
		self.ast = parse_file(self.filepath, use_cpp=True, cpp_path='xc16-gcc', cpp_args=args)

	def defined_functions(self, criterion=lambda x: True):
		visitor = FunctionDefinitionRecorder()
		visitor.visit(self.ast)

		return filter(criterion, visitor.defined_functions)


#!/usr/bin/env python

import sys
import os.path
import os
import intelhex

from pymomo.proteus.project import Project
from pymomo.mib.descriptor import MIBDescriptor
from pymomo.mib.block import MIBBlock
from pymomo.mib.api import MIBAPI
from pymomo.mib import config12
from pymomo.mib.reflash import *
from pymomo.utilities import build
from pymomo.utilities.paths import MomoPaths
import cmdln
from colorama import Fore, Style
import pyparsing

class MIBTool(cmdln.Cmdln):
	name = 'mibtool'

	@cmdln.option('-o', '--output', action='store',  help='Output .pdsprj file to create (without extension)')
	@cmdln.option('-t', '--template', action='store', help='Template project with a Pic24F16KA101 firmware project defined')
	@cmdln.option('-p', '--proc', action='store', default='pic24', choices=['pic24', 'pic12'], help="Processor type choose from pic24 or pic12")
	def do_proteus(self, subcmd, opts, project):
		"""${cmd_name}: Generate a proteus vsm project for this board file.

		Create a Proteus VSM 8 project file, copying all of the source files in 
		to the project file and setting the parameters appropriately for the 
		project to compile out of the box in VSM.  A template project file is 
		required so that we can copy the appropriate settings into the created 
		project file.  It should be an empty project created by Proteus VSM 
		with the correct processor type and a firmware project created with 
		no files added.  If any files are present they will be overwritten.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		self.assert_args(opts, ['template', 'output'])

		if not os.path.exists(opts.template):
			self.error("template project %s does not exist" % opts.template)

		if not os.path.isdir(project):
			self.error("You must pass a directory to mibtool proteus <project directory>")

		proj = Project([project], opts.proc, opts.template)
		proj.create(opts.output)

	def do_build(self, argv):
		"""${cmd_name}: Build the mib12 project in the current directory
		
		${cmd_usage}
		${cmd_option_list}
		"""

		import pymomo.utilities.invoke
		import SCons.Script

		site_path = os.path.abspath(os.path.join(MoMoPaths().base, 'tools', 'site_scons'))


		all_args = ['mibtool', '--site-dir=%s' % site_path]
		sys.argv = all_args + list(argv[1:])
		SCons.Script.main()

	def do_mib12(self, subcmd, opts, name):
		"""${cmd_name}: Create an empty mib12 application module project

		Create a mib12 application module with the given name.  It will
		apeaar in the momo_modules directory with starter files and a
		default empty endpoint file

		name should have no spaces in it.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		fillvars = {
		'module' : {
			'name': name
		}}

		template = RecursiveTemplate('mib12_module', rename=name)
		template.add(fillvars)
		template.render(output_dir=MomoPaths().modules)

		path = os.path.join(MomoPaths().modules, name)
		print "Created mib12 module named %s at:\n%s" % (name, path)

	@cmdln.option('-c', '--chip', help="Chip type that this reflasher corresponds to")
	@cmdln.option('-o', '--output', help="output hex file to create")
	def do_build_reflash(self, subcmd, opts, stub_file, payload_file):
		"""${cmd_name}: build an application module for reflashing the mib12_executive

		Given a mib12_reflash stub hex file and a new version of the mib12_executive,
		combine them into an application module that will reflash the executive onto
		the chip type passed.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		self.assert_args(opts, ['chip', 'output'])

		stub = self.load_ih(stub_file)
		payload = self.load_ih(payload_file)
		chip = opts.chip

		build_reflasher(stub, payload, chip)

		stub.write_hex_file(opts.output)

	@cmdln.option('-c', '--chip', help="Chip type that this reflasher corresponds to")
	@cmdln.option('-o', '--output', help="output hex file to create")
	def do_extract_exec(self, subcmd, opts, stub_file):
		"""${cmd_name}: extract the mib12_executive from a reflash module

		Given a mib12_reflash hex file with a payload, extract that payload and
		save it into output.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		self.assert_args(opts, ['chip', 'output'])

		stub = self.load_ih(stub_file)
		chip = opts.chip

		load = extract_reflasher(stub, chip)
		load.write_hex_file(opts.output)

	@cmdln.option('-o', '--output', action='store', default=None, help="Output direcory to place the command_map.asm file in")
	def do_gen(self, subcmd, opts, mibfile):
		"""${cmd_name}: Compile a .mib file into command_map.asm

		Create a command_map.asm file specifying information about this mib 
		application module by compiling the *.mib file passed in.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		try:
			d = MIBDescriptor(mibfile)
		except pyparsing.ParseException as e:
			self.error(str(e))

		if opts.output is None:
			self.error("You must specify an output directory")
		if not os.path.isdir(opts.output):
			self.error("Invalid output directory specified")

		d.get_block().create_asm(opts.output)

	def do_checksum(self, subcmd, opts, hexfile):
		"""${cmd_name}: Compute an 8-bit checksum for the mib12 app module

		The application module must have a valid mib block so that we can 
		extract the hw type in order to properly calculate where to start 
		computing the checksum.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		#Load the chip type from the mib block and use that to extract the application
		#rom range.
		try:
			b = MIBBlock(hexfile)
			aliases, settings = build.load_chip_info(b.chip_name)
			info = config12.MIB12Processor('unnamed', settings)
			
			start,end = info.app_rom
			print start
			print end
		except:
			self.error(str(sys.exc_info()[1]))

		ih = intelhex.IntelHex(hexfile)
		ih.padding = 0xFF

		check = 0
		for i in xrange(start, end+1):
			low = ih[i*2]
			high = ih[i*2+1]
			high = high & 0b00111111
			check += low + high
			check &= 0xFF

		check = check & 0xFF

		print "\nChecksum:"
		print "hex: 0x%X" % check
		print "dec: %d" % check
		print "bin: %s" % bin(check)

	def do_dump(self, subcmd, opts, hexfile):
		"""${cmd_name}: Dump the MIB Block from a hex file and verify it.

		Given a hex file that should be a valid MIB12 application module, dump 
		the MIB Block, validate the information stored in it and dump it to 
		stdout.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		try:
			b = MIBBlock(hexfile)
			print str(b)
		except:
			self.error(str(sys.exc_info()[1]))

	@cmdln.option('-c', '--chip', help="Chip type that this hex file corresponds to")
	def do_api(self, subcmd, opts, hexfile):
		"""${cmd_name}: Dump the MIB API region from the hex file and verify it 

		Given a hex file that should be a valid MIB12 executive module, 
		validate the information stored in it and dump it to stdout.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		self.assert_args(opts, ['chip'])

		try:
			print "\nDumping file: %s" % hexfile
			api = MIBAPI(hexfile, opts.chip)
			api.print_api()
		except:
			self.error(str(sys.exc_info()[1]))

	def load_ih(self, hexfile):
		try:
			ih = intelhex.IntelHex16bit(hexfile)
		except IOError as e:
			self.error(str(e))

		ih.padding = 0x3FFF

		return ih

	def assert_args(self, opts, args):
		for arg in args:
			if not hasattr(opts, arg) or getattr(opts, arg) is None:
				self.error("You must specify an argument for %s" % arg)

	def error(self, text):
		print Fore.RED + "Error Occurred: " + Style.RESET_ALL + text
		sys.exit(1)

def main():
	mibtool = MIBTool()
	return mibtool.main()


#descriptor.py
#Define a Domain Specific Language for specifying MIB endpoints

from pyparsing import Word, Regex, nums, Literal, Optional, Group
from handler import MIBHandler
import sys
import os.path
from pymomo.utilities.paths import MomoPaths
import block

#DSL for mib definitions
#Format:
#feature <i>
#[j:] _symbol(n [ints], yes|no)
#[j+1:] _symbol2(n [ints], yes|no)
#The file should contain a list of statements beginning with a feature definition
#and followed by 1 or more command definitions.  There may be up to 128 feature definitions
#each or which may have up to 128 command definitions.  Whitespace is ignored.  Lines starting
#with # are considered comments and ignored.
symbol = Regex('[_a-zA-Z][_a-zA-Z0-9]*')
filename = Regex('[_a-zA-Z][_a-zA-Z0-9]*\.mib')
strval = Regex('"[_a-zA-Z0-9]+"')
number = (Word(nums).setParseAction(lambda s,l,t: [int(t[0])])) | symbol
ints = number('num_ints') + Optional(Literal('ints') | Literal('int'))
has_buffer = (Literal('yes') | Literal('no')).setParseAction(lambda s,l,t: [t[0] == 'yes'])
comma = Literal(',').suppress()
left = Literal('(').suppress()
right = Literal(')').suppress()
colon = Literal(':').suppress()
comment = Literal('#')

assignment_def = symbol("variable") + "=" + (number('value') | strval('value')) + ';'
cmd_def = number("cmd_number") + colon + symbol("symbol") + left + ints + comma + has_buffer('has_buffer') + right + ";"
feat_def = Literal("feature") + number("feature_number")
include = Literal("#include") + left + filename("filename") + right

statement = include | cmd_def | feat_def | comment | assignment_def

class MIBDescriptor:
	"""
	Class that parses a .mib file which contains a DSL specifying the valid MIB endpoints
	in a MIB12 module and can output an asm file containing the proper MIB command map
	for that architecture.
	"""
	def __init__(self, filename):
		self.curr_feature = -1
		self.features = {}
		self.variables = {}

		self._parse_file(filename)

		for feature in self.features.keys():
			self._validate_feature(feature)
			self.features[feature] = [self.features[feature][i] for i in sorted(self.features[feature].keys())]

		self._validate_information()
	def _parse_file(self, filename):
		with open(filename, "r") as f:
			for l in f:
				line = l.lstrip().rstrip()

				if line == "":
					continue

				self._parse_line(line)

	def _add_feature(self, feature_var):
		feature = self._parse_number(feature_var)

		self.features[feature] = {}
		self.curr_feature = feature

	def _add_cmd(self, num, symbol, num_ints, has_buffer):
		handler = MIBHandler.Create(symbol=symbol, ints=num_ints, has_buffer=has_buffer)
		
		self.features[self.curr_feature][num] = handler

	def _parse_cmd(self, match):
		symbol = match['symbol']

		if self.curr_feature < 0:
			raise ValueError("MIB Command specified without first declaring a feature.")

		num = self._parse_number(match['cmd_number'])

		has_buffer = match['has_buffer']
		num_ints = match['num_ints']

		self._add_cmd(num, symbol, num_ints=num_ints, has_buffer=has_buffer)

	def _parse_include(self, match):
		filename = match['filename']

		folder = MomoPaths().config

		path = os.path.join(folder, filename)
		self._parse_file(path)

	def _parse_number(self, number):
		if isinstance(number, int):
			return number

		if number in self.variables:
			return self.variables[number]

		raise ValueError("Reference to undefined variable %s" % number)

	def _parse_assignment(self, match):
		var = match['variable']
		val = match['value']
		if isinstance(val, basestring) and val[0] == '"':
			val = val[1:-1]
		else:
			val = self._parse_number(match['value'])

		self.variables[var] = val

	def _parse_line(self, line):
		matched = statement.parseString(line)

		if 'feature_number' in matched:
			self._add_feature(matched['feature_number'])
		elif 'symbol' in matched:
			self._parse_cmd(matched)
		elif 'filename' in matched:
			self._parse_include(matched)
		elif 'variable' in matched:
			self._parse_assignment(matched)

	def _validate_feature(self, feature):
		curr_index = 0

		for i in sorted(self.features[feature]):
			if i != curr_index:
				raise ValueError("Feature %d has nonsequential command.  Must start at 0 and have no gaps. Command was %d, expected %d." % (feature, i, curr_index))

			curr_index += 1

	def _validate_information(self):
		"""
		Validate that all information has been filled in
		"""

		needed_variables = ["ModuleName", "ModuleFlags", "ModuleType"]

		for var in needed_variables:
			if var not in self.variables:
				raise ValueError("Needed variable %s was not defined in mib file." % var)

		#Make sure ModuleName is <= 7 characters
		if len(self.variables["ModuleName"]) > 7:
			raise ValueError("ModuleName ('%s') too long, must be less than 8 characters." % self.variables["ModuleName"])

		if not isinstance(self.variables["ModuleType"], int):
			raise ValueError("ModuleType ('%s') must be an integer." % str(self.variables['ModuleType']))

		if not isinstance(self.variables["ModuleFlags"], int):
			raise ValueError("ModuleFlags ('%s') must be an integer." % str(self.variables['ModuleFlags']))

		self.variables["ModuleName"] = self.variables["ModuleName"].ljust(7)

	def get_block(self):
		"""
		Create a MIB Block based on the information in this descriptor
		"""

		mib = block.MIBBlock()

		for feat, cmds in self.features.iteritems():
			for i,cmd in enumerate(cmds):
				mib.add_command(feat, i, cmd)

		mib.set_variables(	name=self.variables["ModuleName"], flags=self.variables["ModuleFlags"],
							type=self.variables["ModuleType"])

		return mib


#gpsim_runner.py

import SCons.Builder
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..'))
from pymomo.utilities import config


run_gpsim = SCons.Builder.Builder(
	action = 'gpsim -c $SOURCE -i > /dev/null'
	)

def generate(env):
	env['BUILDERS']['gpsim_run'] = run_gpsim

def exists(env):
	return 1


from pymomo.utilities import typedargs
from pymomo.utilities.typedargs import type_system
import unittest
from nose.tools import *
import os.path
from pymomo.exceptions import *

def test_type_injection():
	import extra_type_package.extra_type as typeobj

	assert not type_system.is_known_type("test_injected_type1")

	type_system.inject_type("test_injected_type1", typeobj)
	assert type_system.is_known_type("test_injected_type1")

def test_external_module_injection():
	"""
	Test type injection from an external python module
	"""

	path = os.path.join(os.path.dirname(__file__), 'extra_types')

	assert not type_system.is_known_type('new_type')
	type_system.load_external_types(path)
	assert type_system.is_known_type('new_type')

def test_external_package_injection():
	"""
	Test type injection from an external python package
	"""

	path = os.path.join(os.path.dirname(__file__), 'extra_type_package')

	assert not type_system.is_known_type('new_pkg_type')
	type_system.load_external_types(path)
	assert type_system.is_known_type('new_pkg_type')

@raises(ArgumentError)
def test_external_package_injection_failure():
	"""
	Test type injection raises error from nonexistant path
	"""

	path = os.path.join(os.path.dirname(__file__), 'extra_type_package_nonexistant')
	type_system.load_external_types(path)



from command import Command
from pymomo.commander.exceptions import *
import base64

class RPCCommand (Command):
	def __init__(self, address, feature, command, *args):
		"""
		Create an RPC command 
		"""

		self.addr = int(address)
		self.feat = int(feature)
		self.cmd = int(command)
		self.spec = 0

		self.args = args
		self.result = ""

	def __str__(self):
		args = self._format_args()
		header = bytearray(4)
		header[0] = self.addr
		header[1] = self.feat
		header[2] = self.cmd
		header[3] = self.spec

		packet = header + args

		cmd = "binrpc %s" % base64.standard_b64encode(packet)
		return cmd

	def _convert_int(self, arg):
		out = bytearray(2)

		out[0] = arg & 0xFF
		out[1] = (arg & 0xFF00) >> 8;

		converted = out[0] | (out[1] << 8)

		if converted != arg:
			raise ValueError("Integer argument was too large to fit in an rpc 16 bit int: %d" % arg)

		return out

	def _pack_arg(self, arg):
		if isinstance(arg, int) or isinstance(arg, long):
			return self._convert_int(arg), False
		elif isinstance(arg, bytearray):
			return arg, True
		elif isinstance(arg, basestring):
			return bytearray(arg), True

		raise ValueError("Unknown argument type could not be converted for rpc call.")

	def _format_args(self):
		fmtd = bytearray()

		num_ints = 0
		num_bufs = 0

		for arg in self.args:
			a,is_buf = self._pack_arg(arg)
			fmtd += a

			if is_buf:
				num_bufs += 1
				buff_len = len(a)

			if not is_buf:
				if num_bufs != 0:
					raise ValueError("Invalid rpc parameters, integer came after buffer.")

				num_ints += 1

		if num_bufs > 1:
			raise ValueError("You must pass at most 1 buffer. num_bufs=%d" % num_bufs)

		if num_ints > 4:
			raise ValueError("You must pass at most 4 integers. num_ints=%d" % num_ints)

		if len(fmtd) > 20:
			raise ValueError("Arguments are greater then the maximum mib packet size, size was %d" % len(fmtd))

		if len(fmtd) < 20:
			fmtd += bytearray(20 - len(fmtd))

		#Calculate the command type spec
		spec = 0

		if num_bufs == 1:
			spec = 1 << 7
			spec |= buff_len & 0b11111

		spec |= (num_ints&0b11) << 5

		self.spec = spec

		return fmtd


	def handle_result(self, stream):
		"""
		rpc command on 
		"""
		status = ord(stream.trans.read())
		self.status = status

		if status == 254:
			buf, term = stream.read_frame()
			self.result = buf
			return buf, term

		if status == 0:
			num_bytes = ord(stream.trans.read())
			buf = stream.trans.read(num_bytes)
			self.result = buf

		seq = stream.trans.read()
		self.status = status

		return self.result, stream.parse_term(seq)

	def parse_result(self, num_ints, buff):
		parsed = {'ints':[], 'buffer':"", 'status': self.status, 'error': 'No Error'}

		if self.status == 254:
			parsed['error'] = self.result
			return parsed
		elif self.status != 0:
			if self.status == 1:
				parsed['error'] = 'Unsupported Command'
			elif self.status == 2:
				parsed['error'] = 'Wrong Parameter Type'
			elif self.status == 3:
				parsed['error'] = 'Parameter Too Long'
			elif self.status == 4:
				parsed['error'] = 'Checksum Error'
			elif self.status == 6:
				parsed['error'] = 'Unknown Error'
			elif self.status == 7:
				parsed['error'] = 'Callback Error'
			elif self.status == 0xFF:
				parsed['error'] = 'Module at address ' + str(self.addr) + ' not found.'
			else:
				parsed['error'] = 'Unrecognized MIB status code'
			return parsed

		#Otherwise, parse the results according to the type information given
		size = len(self.result)

		if size < 2*num_ints:
			raise RPCException(300, 'Return value too short to unpack : %s' % self.result)
		elif buff == False and size != 2*num_ints:
			raise RPCException(301, 'Return value does not match return type: %s' % self.result)

		for i in xrange(0, num_ints):
			low = ord(self.result[2*i])
			high = ord(self.result[2*i + 1])
			parsed['ints'].append((high << 8) | low)

		if buff:
			parsed['buffer'] = self.result[2*num_ints:]

		return parsed



from pymomo.exceptions import *
import struct
import datetime

class MessageEntry:
	Streams = {
	1: "CRITICAL",
	2: "DEBUG",
	3: "REMOTE",
	4: "AUDIT",
	5: "PERFORMANCE"
	}

	def __init__(self, buf):
		if len(buf) != 15:
			raise ValidationError("Wrong length for new log entry (typecode 0), should be 15", length=len(buf))

		stream, timestamp, msg, remote_source, remote_seq, res1, res2 = struct.unpack('<BLLBBHH', buf)

		self.stream = stream

		#Two important notes about this
		#1: the magic number 946684800 is the difference between the UNIX Epoch and 
		#Jan 1, 2000 12:00am in seconds since the RTCC in the PICs counts from 0 at
		#Jan 1, 2000 12:00am
		#2: utcfromtimestamp is used rather than fromtimestamp because fromtimestamp
		#assumes that the timestamp is in UTC and applies a timezone correction to your
		#current timezone.  utcfromtimestamp does not.  Since times are in local time
		#in the PIC, we don't want to apply any timezone correction and just display the
		#data as it is in the momo.
		self.timestamp = datetime.datetime.utcfromtimestamp(timestamp + 946684800)
		self.msg = msg
		self.remote_source = remote_source
		self.remote_seq = remote_seq

		if self.stream not in MessageEntry.Streams:
			raise ValidationError("Unknown stream number for this log entry", stream=self.stream)

	def stream_name(self):
		return MessageEntry.Streams[self.stream]

	def __str__(self):
		msg = format(self.msg, "08X")
		return "%s (%s) 0x%s" % (self.stream_name(), self.timestamp, msg)


class RawLogEntry:
	"""
	A generic entry in the MoMo syslog logging system
	that can be created from a buffer dumped via an RPC
	call to the momo controller hardware.
	"""

	NewLogEntryType = 0
	ContinuationType = 1
	BlobType = 2
	IntegerListType = 3
	StringType = 4
	RemoteLogEndType = 5

	Parsers = {
		NewLogEntryType: MessageEntry
	}

	def __init__(self, buffer):
		if len(buffer) != 16:
			raise ValidationError("Buffer must be of length sizeof(GenericLogEntry) = 16 bytes", buffer=buffer)

		header = ord(buffer[0])
		self.data = buffer[1:]

		self._parse_header(header)

	def _parse_header(self, header):
		typecode = header & 0xF
		length = (header >> 4) & 0xF

		if length > 15:
			raise ValidationError("Buffer length too long in GenericLogEntry", length=length, typecode=typecode)

		self.data = self.data[:length]
		self.length = length
		self.typecode = typecode

		if typecode < 0 or typecode > 5:
			raise ValidationError("Invalid typecode in GenericLogEntry", typecode=typecode)

	def merge(self, other):
		if other.typecode != RawLogEntry.ContinuationType:
			raise ValidationError("You can only combine log entries that are ContinuationType", typecode=other.typecode)

		if self.typecode == RawLogEntry.NewLogEntryType:
			raise ValidationError("You cannot combine a NewLogEntry with another type of log entry")

		self.data += other.data
		self.length += other.length

	def __str__(self):
		if self.typecode == RawLogEntry.NewLogEntryType:
			return str(MessageEntry(self.data))

		dat = " ".join([format(ord(x), "02X") for x in self.data])
		rep = "LogEntry type %d (%s)" % (self.typecode, dat)
		return rep



from pymomo.commander.exceptions import *
import struct
import math

class PerformanceCounter:
	def __init__(self, buffer, type="timer", name="Unnamed"):
		if len(buffer) != 20:
			raise TypeException("PerformanceCounter initialized from buffer of length (%d), should be 20." % len(buffer))

		count, mean, minval, maxval, accum = struct.unpack("<LLLLL", buffer)

		self.count = count
		self.mean = mean
		self.accum = accum
		self.min = minval
		self.max = maxval
		self.name = name
		self.type = type

	def format_value(self, val):
		if self.type == "timer":
			us = val*0.25
			if us < 1e3:
				time = "%.2f us" % us
			elif us < 1e6:
				time = "%.2f ms" % (us/1000.0,)
			else:
				time = "%.2f s" % (us/1e6,)

			return "%s (%d cycles)" % (time, val)

		return str(val)

	def display(self):
		print "Performance Counter (%s)" % str(self.name)
		print "Number of Events: %d" % self.count
		print "Min: %s" % self.format_value(self.min)
		print "Max: %s" % self.format_value(self.max)
		print "Mean: %s" % self.format_value(self.mean)



"""
A package for managing bills of materials and pcb fabrication

Methods and objects for creating and pricing bills of materials
as well as automatically generating gerber files for pcb fabrication.

- The CircuitBoard object provides a way to generate BOMs and production
  files for pcb fabrication from ECAD files
- various BOM pricing and matching engines like OctopartMatcher allow you
  to see how much your BOM would cost in different quantities.
"""

_name_ = "pcb"


#Add in required types that we need 
import types
import pymomo.utilities.typedargs
pymomo.utilities.typedargs.type_system.load_type_module(types)

from match_engines import *
from board import CircuitBoard



import SCons.Builder
import utilities
import intelhex

def merge_app(target, source, env):
	"""
	Given two source files, the first being the mib12_executive and the second
	being an application hex, merge the two into a complete application hex
	"""

	chip = env['CHIP']

	exec_end = chip.exec_rom[1]*2

	execfile = intelhex.IntelHex(str(source[0]))
	appfile = intelhex.IntelHex(str(source[1]))

	execfile.merge(appfile[exec_end+2:], overlap='replace')
	execfile.tofile(str(target[0]), format='hex')

_merge_app = SCons.Builder.Builder(
	action = merge_app
	)

def generate(env):
	env['BUILDERS']['merge_mib12_app'] = _merge_app

def exists(env):
	return 1




import unittest
import os.path
from nose.tools import *
from pymomo.exceptions import *
from pymomo.pcb.partcache import PartCache
import tempfile
import shutil
import atexit

class TestCache(unittest.TestCase):
	"""
	Test to make sure that caching of part request responses is working.
	"""

	def _create_tempfile(self, src=None):
		"""
		Create a named temporary file that is a copy of the src file
		and register for it to be deleted at program exit.
		"""
		dst = tempfile.NamedTemporaryFile(delete=False)

		if src is not None:
			with open(src, "r+b") as f:
				shutil.copyfileobj(f, dst)

		dst.close()

		atexit.register(os.remove, dst.name)
		return dst.name

	def setUp(self):
		self.filled_cachefile = self._create_tempfile(os.path.join(os.path.dirname(__file__), 'eagle', 'pcb_part_cache.db'))
		self.empty_cachefile = self._create_tempfile()

	def test_create_cache(self):
		pc = PartCache(cache=self.empty_cachefile)

		assert pc.size() == 0

		val = {'test':'test', 'hello': 2}
		pc.set('hello', val)
		assert pc.size() == 1

		stored = pc.get('hello')
		assert val == stored

	def test_expire_parts(self):
		"""
		Make sure parts are expired like they should be
		"""

		pc1 = PartCache(cache=self.filled_cachefile, no_expire=True)
		assert pc1.size() > 0

		pc2 = PartCache(cache=self.filled_cachefile, no_expire=False, expiration=1)
		assert pc2.size() == 0



#convert.py
#Convert PIC24 hex files into a form that can be programmed for recovery
#Each pic24 instruction is 3 bytes.  XC16 pads this to 4 bytes.  We can't
#have it padded since we need to program it into the device directly

import intelhex as ih

def unpad_pic24_hex(hexfile):
	"""
	Given a pic24 hex file, return an intel hex object that is not padded
	"""

	f = ih.IntelHex(hexfile)
	out = ih.IntelHex()

	i = 0
	j = 0


	if (f.maxaddr() +1) % 4 != 0:
		raise ValueError("Cannot pack PIC24 hex file since its length is not a multiple of 4")

	for i in xrange(0, f.maxaddr()+1, 4):
		out[j+0] = f[i+0]
		out[j+1] = f[i+1]
		out[j+2] = f[i+2]
		j += 3

	return out

def pad_pic24_hex(f):
	out = ih.IntelHex()

	i = 0
	j = 0

	if (f.maxaddr() + 1) % 3 != 0:
		raise ValueError("PIC24 hex file did not have length of a multiple of 3, was not correctly packed.")
	
	for i in xrange(0, f.maxaddr()+1, 3):
		out[j+0] = f[i+0]
		out[j+1] = f[i+1]
		out[j+2] = f[i+2]
		out[j+3] = 0x00

		j += 4

	return out


#unit_price.py
#An object encapsulating a list of pricebreaks

from decimal import Decimal, getcontext

def convert(arg, **kwargs):
	if isinstance(arg, UnitPrice):
		return arg

	if isinstance(arg, basestring):
		raise ValueError("Converting to price_list from string is not yet supported")

	return UnitPrice(**arg)

def default_formatter(arg, **kwargs):
	return str(arg)

class UnitPrice:
	def __init__(self, price, offer):
		self.price = price
		self.offer = offer		

	def __str__(self):
		tp = Decimal('0.01')
		pricestr = str(self.price.quantize(tp))
		return "$%s from %s as %s" % (pricestr, self.offer.seller, self.offer.packaging)



#annotate.py
from decorator import decorator
from pymomo.exceptions import *
import inspect
from typeinfo import type_system
from collections import namedtuple
import sys

class BasicContext(dict):
	pass

def _check_and_execute(f, *args, **kwargs):
	"""
	Check the type of all parameters with type information, converting 
	as appropriate and then execute the function.
	"""

	convargs = []
	spec = inspect.getargspec(f)

	#Convert and validate all arguments
	for i in xrange(0, len(args)):
		arg = spec.args[i]
		val = _process_arg(f, arg, args[i])
		convargs.append(val)

	convkw = {}
	for key, val in kwargs:
		convkw[key] = _process_arg(f, key, val)

	#Ensure that only MoMoException subclasses are passed by the caller
	try:
		retval = f(*convargs, **convkw)
	except MoMoException:
		raise
	except Exception as unknown:
		raise APIError(str(unknown.args)), None, sys.exc_info()[2]

	return retval

def _process_arg(f, arg, value):
	"""
	Ensure that value is a valid argument for the named 
	parameter arg based on the annotated type and validator
	information.  Any errors are raised as either 
	ConversionError or ValidationError exceptions.
	"""

	if arg in f.params:
		val = type_system.convert_to_type(value, f.types[arg])
	else:
		val = value

	#Run all of the validators
	try:
		if arg in f.valids:
			for valid in f.valids[arg]:
				valid[0](val, *valid[1])
	except (ValueError,TypeError) as e:
		raise ValidationError(e.args[0], argument=arg, value=val)

	return val

def _parse_validators(type, valids):
	"""
	Given a list of validator names or n-tuples, map the name to 
	a validation function given the type and return a list of 
	validation function, arguments tuples
	"""

	outvals = []

	for val in valids:
		if isinstance(val, basestring):
			args = []
		elif len(val) > 1:
			args = val[1:]
			val = val[0]
		else:
			raise ValueError("You must pass either a n-tuple or a string to define a validator")

		name = "validate_%s" % str(val)

		if not hasattr(type, name):
			raise AttributeError("Cannot resolve validator")

		func = getattr(type, name)
		outvals.append((func, args))

	return outvals

def get_spec(f):
	if inspect.isclass(f):
		f = f.__init__

	spec = inspect.getargspec(f)

	if spec.defaults is None:
		numreq = len(spec.args)
	else:
		numreq = len(spec.args) - len(spec.defaults)

	#If the first argument is self, don't return it
	start = 0
	if numreq > 0 and spec.args[0] == 'self':
		start = 1

	reqargs = spec.args[start:numreq]
	optargs = set(spec.args[numreq:])

	return reqargs, optargs

def spec_filled(req, opt, pos, kw):
	left = filter(lambda x: x not in kw, pos)
	left = req[len(left):]

	if len(left) == 0:
		return True

	return False

def get_signature(f):
	"""
	Return the pretty signature for this function:
	foobar(type arg, type arg=val, ...)
	"""

	name = f.__name__

	if inspect.isclass(f):
		f = f.__init__

	spec = inspect.getargspec(f)
	num_args = len(spec.args)

	num_def = 0
	if spec.defaults is not None:
		num_def = len(spec.defaults)
	
	num_no_def = num_args - num_def

	args = []
	for i in xrange(0, len(spec.args)):
		typestr = ""
		if i == 0 and spec.args[i] == 'self':
			continue

		if spec.args[i] in f.types:
			typestr = "%s " % f.types[spec.args[i]]

		if i >= num_no_def:
			default = str(spec.defaults[i-num_no_def])
			if len(default) == 0:
				default="''"

			args.append("%s%s=%s" % (typestr, str(spec.args[i]), default))
		else:
			args.append(typestr + str(spec.args[i]))

	return "%s(%s)" % (name, ", ".join(args))

def print_help(f):
	"""
	Print usage information about a context or function.

	For contexts, just print the context name and its docstring
	For functions, print the function signature as well as its
	argument types.
	"""

	if isinstance(f, BasicContext):
		name = context_name(f)

		print "\n" + name + "\n"
		doc = inspect.getdoc(f)
		if doc is not None:
			doc = inspect.cleandoc(doc)
			print doc

		return

	sig = get_signature(f)
	doc = inspect.getdoc(f)
	if doc is not None:
		doc = inspect.cleandoc(doc)

	print "\n" + sig + "\n"
	if doc is not None:
		print doc

	if inspect.isclass(f):
		f = f.__init__

	print "\nArguments:"
	for key in f.params.iterkeys():
		type = f.types[key]
		desc = ""
		if key in f.param_descs:
			desc = f.param_descs[key]

		print " - %s (%s): %s" % (key, type, desc)

def print_retval(f, value):
	if hasattr(f, 'typed_retval') and f.typed_retval == True:
		print type_system.format_return_value(f, value)
		return

	if not hasattr(f, 'retval'):
		print str(value)

	elif f.retval.printer[0] is not None:
		f.retval.printer[0](value)
	elif f.retval.desc != "":
		print "%s: %s" % (f.retval.desc, str(value))

def find_all(container):
	if isinstance(container, dict):
		names = container.keys()
	else:
		names = dir(container)
	
	context = BasicContext()

	for name in names:
		#Ignore _ and __ names
		if name.startswith('_'):
			continue

		if isinstance(container, dict):
			obj = container[name]
		else:
			obj = getattr(container, name)

		#Check if this is an annotated object that should be included.  Check the type of
		#annotated to avoid issues with module imports where someone did from annotate import *
		#into the module causing an annotated symbol to be defined as a decorator
		
		#If we are in a dict context then strings point to lazily loaded modules so include them
		#too.
		if isinstance(container, dict) and isinstance(obj, basestring):
			context[name] = obj
		elif hasattr(obj, 'annotated') and isinstance(getattr(obj, 'annotated'), int):
			context[name] = obj

	return context

def context_from_module(module):
	"""
	Given a module, create a context from all of the top level annotated
	symbols in that module.  
	"""

	con = find_all(module)

	if hasattr(module, "__doc__"):
		setattr(con, "__doc__", module.__doc__)

	if hasattr(module, "_name_"):
		name = module._name_
	else:
		name = module.__name__

	setattr(con, '_annotated_name', name)
	setattr(con, 'context', True)

	con = annotated(con)

	return name, con


def check_returns_data(f):
	if hasattr(f, 'typed_retval') and f.typed_retval == True:
		return True

	if not hasattr(f, 'retval'):
		return False

	return f.retval.data

#Decorators
def param(name, type, *validators, **kwargs):
	def _param(f):
		f = annotated(f)

		f.params[name] = type_system.get_type(type)
		f.types[name] = type
		f.valids[name] = _parse_validators(f.params[name], validators)

		if 'desc' in kwargs:
			f.param_descs[name] = kwargs['desc']

		return decorator(_check_and_execute, f)

	return _param

def returns(desc=None, printer=None, data=True):
	"""
	Specify how the return value of this function should be handled

	If data == True, then this function just returns data and does 
	not return a context so that the context for future calls remains
	unchanged.  
	"""

	def _returns(f):
		annotated(f)

		f.retval = namedtuple("ReturnValue", ["desc", "printer", "data"])
		f.retval.desc = desc
		f.retval.printer = (printer,)
		f.retval.data = data

		return f

	return _returns

def return_type(type, formatter=None):
	"""
	Specify that this function returns a typed value

	type must be a type known to the MoMo type system and formatter
	must be a valid formatter for that type
	"""

	def _returns(f):
		annotated(f)
		f.typed_retval = True
		f.retval_type = type_system.get_type(type)
		f.retval_typename = type
		f.retval_formatter = formatter

		return f

	return _returns

def context(name=None):
	"""
	Declare that a class defines a MoMo context for use with the momo function for discovering
	and using functionality.
	"""

	def _context(cls):
		annotated(cls)
		cls.context = True

		if name is not None:
			cls._annotated_name = name
		else:
			cls._annotated_name = cls.__name__

		return cls

	return _context

def finalizer(f):
	"""
	Indicate that this function destroys the context in which it is invoked, such as a quit method
	on a subprocess or a delete method on an object.
	"""

	f = annotated(f)
	f.finalizer = True
	return f

def context_name(context):
	"""
	Given a context, return its proper name
	"""

	if hasattr(context, "_annotated_name"):
		return context._annotated_name
	elif inspect.isclass(context):
		return context.__class__.__name__

	return str(context)

def takes_cmdline(f):
	f = annotated(f)
	f.takes_cmdline = True

	return f
	
def annotated(f):
	if not hasattr(f, 'params'):
		f.params = {}
	if not hasattr(f, 'valids'):
		f.valids = {}
	if not hasattr(f, 'types'):
		f.types = {}
	if not hasattr(f, 'param_descs'):
		f.param_descs = {}

	f.annotated = True
	f.finalizer = False
	f.takes_cmdline = False
	return f

def short_description(f):
	"""
	Given an object with a docstring, return the first line of the docstring
	"""

	doc = inspect.getdoc(f)
	if doc is not None:
		doc = inspect.cleandoc(doc)
		lines = doc.splitlines()
		return lines[0]

	return ""


#xc16_linker.py
#SCons Builder Actions for the XC16 PIC assembler

import SCons.Builder
import SCons.Action
import os.path
import sys
import utilities

sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..'))
from pymomo.utilities.paths import convert_path

def get_map_file(targetdir, env):
	return os.path.join(targetdir, env['ARCH'].output_name() + '.map')

def xc16_generator(source, target, env, for_signature):
	"""
	Create an XC16 command line using the parameter defined in 
	the environment
	"""

	arch = env['ARCH']
	targetdir = str(target[0].get_dir())

	#Build up the command line
	args = ['xc16-gcc']
	args.extend(['-mcpu=%s' % arch.property('chip')])
	args.extend([str(x) for x in source])
	args.extend(['-o %s' % (str(target[0]))])
	args.extend(utilities.build_libdirs(arch.property('libdirs', default=[])))
	args.extend(utilities.build_staticlibs(arch.property('libraries', default=[]), arch))
	args.extend(['-Wl,-Map=%s' % (get_map_file(targetdir, env))])
	args.extend(arch.property('ldflags', default=[]))

	linker_script = arch.property('linker', default=None)
	if linker_script is not None:
		args.append('-T%s' % linker_script)

	return SCons.Action.Action(" ".join(args), "Linking %s" % str(target[0]))

def xc16_emitter(target, source, env):
	targetdir = str(target[0].get_dir())
	target += [get_map_file(targetdir, env)]
	return target, source

_xc16_obj = SCons.Builder.Builder(
	generator = xc16_generator,
	emitter = xc16_emitter,
	suffix='.o')

def generate(env):
	env['BUILDERS']['xc16_ld'] = _xc16_obj

def exists(env):
	return 1


#Transport layer for momo commander.  Should be able to send a stream of bytes
#and list whether any fsu devices are presently connected

from pymomo.commander.exceptions import *

class Transport:
	def read_until(self, chars):
		buffer = ""

		while True:
			c = self.read()
			if len(c) == 0:
				raise TimeoutException("Transport.read_until, looking for %s" % chars)
						
			for x in chars:
				if ord(c) == x:
					return (buffer, c)

			buffer += c


#symbols.py
#Create a list of the symbols in an xc8 *.sym file
#Also allows looking up an address to find the corresponding symbol and
#saving and merging symbol tables

import sys
import json
import bisect
import os.path

class XC8SymbolTable:
	def __init__(self, path):
		ext = os.path.splitext(path)[1]

		if ext == '.sym':
			self._parse_symtab(path)
			self._calc_sizes()
		elif ext == '.stb':
			with open(path, "r") as f:
				keys = json.load(f)
				self.known_functions = keys['functions']
				self.global_vars = keys['globals']
				self.local_vars = keys['locals']
		else:
			raise ValueError("Invalid symbol file with extension != [sym|stb]: %s" % path)
		
		self._build_lookup()

	def _parse_symtab(self, path):
		print path
		lines = [line.strip() for line in open(path)]

		self.symtab = {}
		self.sects = {}
		self.known_functions = []
		self.local_vars = []
		self.global_vars = []

		for line in lines:
			#The data after the segments section does not contain any symbols.
			if line == "%segments":	
				break

			fields = line.split(' ')

			if len(fields) != 5:
				raise ValueError("File did not have a valid format, too many fields in line:\n%s\n" % line)

			name = fields[0]
			addr = int(fields[1], 16)
			sect = fields[3]

			if name.startswith('__end_of'):
				self.known_functions.append((name[8:], addr))
				continue
			elif name.startswith('__size_of'):
				continue

			#Process ram symbols as either local or global variables
			if sect != 'CODE':
				self._process_variable(name, addr)

			self.symtab[name] = (addr, sect)

			#Keep track of which symbols are in which sections
			if sect not in self.sects:
				self.sects[sect] = [name]
			else:
				self.sects[sect].append(name)

	def _process_variable(self, name, addr):
		if '@' in name:
			self.local_vars.append((name, addr))
		else:
			self.global_vars.append((name, addr))

	def merge(self, path):
		"""
		Merge another symbol table into this one
		"""

		other = XC8SymbolTable(path)

		self.known_functions += other.known_functions
		self._build_lookup()

	def _calc_sizes(self):
		"""
		Calculate the sizes of all known_functions: functions with a 
		symbol defining the end of the function, i.e. __end_of_foo
		
		Converts the known_functions list in 3-tuples with (name, starting_address, size)
		"""

		self.known_functions = map(lambda x: (x[0], self.symtab[x[0]][0], x[1] - self.symtab[x[0]][0]), self.known_functions)
		self.total_size = reduce(lambda x,y: x+y, map(lambda x: x[2], self.known_functions))

	def _build_lookup(self):
		self.known_functions = sorted(self.known_functions, key=lambda x:x[1])
		self.lookup = map(lambda x: x[1], self.known_functions)

	def map_address(self, addr):
		"""
		Given an address, find the function and offset within that function that it belongs to
		"""

		i = bisect.bisect_right(self.lookup, addr)
		i -= 1

		if i < 0:
			return None

		offset = addr - self.known_functions[i][1]

		if offset < self.known_functions[i][2]:
			return (self.known_functions[i][0], offset)

		return None

	def _check_c_name(self, var):
		if '$' in var or '?' in var:
			return False

		return True

	def _gen_variable_def(self, var):
		if not self._check_c_name(var[0]):
			return None

		line = '#define %s_variable 0x%x\n' % (var[0], var[1])
		return line

	def _gen_define(self, func):
		if not self._check_c_name(func[0]):
			return None

		line = '#define %s_address 0x%x\n' % (func[0], func[1])
		return line

	def _gen_c_call(self, func):
		if not self._check_c_name(func[0]):
			return None

		#Use fcall in case call is not in the first page of rom
		line = '#define c_call%s() asm("fcall 0x%x")\n' % (func[0], func[1])
		return line 

	def _gen_asm_call(self, func):
		if not self._check_c_name(func[0]):
			return None

		line = '#define asm_call%s() fcall 0x%x\n' % (func[0], func[1])
		return line 

	def generate_h_file(self, path):
		defs = map(lambda x: self._gen_define(x), self.known_functions)
		c_calls = map(lambda x: self._gen_c_call(x), self.known_functions)
		asm_calls = map(lambda x: self._gen_asm_call(x), self.known_functions)
		global_vars = map(lambda x: self._gen_variable_def(x), self.global_vars)

		defs = filter(lambda x: x is not None, defs)
		c_calls = filter(lambda x: x is not None, c_calls)
		asm_calls = filter(lambda x: x is not None, asm_calls)
		global_vars = filter(lambda x: x is not None, global_vars)

		with open(path, "w") as f:
			f.write('//Autogenerated Symbol Header File (do not modify)\n')
			f.write('//Known Functions (addresses and call macros)\n')
			f.writelines(defs)
			f.writelines(c_calls)
			f.writelines(asm_calls)
			f.write('\n//Global Variables\n')
			f.writelines(global_vars)

	def generate_stb_file(self, path):
		with open(path, "w") as f:
			json.dump({'functions': self.known_functions, 'globals': self.global_vars, 'locals': self.local_vars}, f)



#!/usr/bin/env python

import sys
import os.path

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from pymomo.sim import Simulator
from pymomo.exceptions import *

def main():
	if len(sys.argv) != 4:
		print "Usage: momo-picunit (pic24super|pic24fpsuper) ELF_FILE OUTPUT_LOG"
		return 1

	model = sys.argv[1]
	elf = sys.argv[2]
	log = sys.argv[3]

	try:
		sim = Simulator('pic24')
		sim.set_param('model', model)
		sim.attach_log()
		sim.load(elf)
		sim.execute()
		sim.wait(10)
		logdata = sim.get_log()
	except MoMoException as e:
		print e.format()
		return 1

	with open(log, "w") as f:
		f.write(logdata)

	return 0


#xc16_compiler.py
#SCons Builder Actions for the XC16 PIC compiler

import SCons.Builder
import SCons.Action
import os.path
import sys
import utilities

sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..'))
from pymomo.utilities.paths import convert_path

def xc16_generator(source, target, env, for_signature):
	"""
	Create an XC16 command line using the parameter defined in 
	the environment
	"""

	arch = env['ARCH']

	#Build up the command line
	args = ['xc16-gcc']
	args.extend(['-mcpu=%s' % arch.property('chip')])
	args.extend(['-c'])
	args.append('"%s"' % str(source[0]))
	args.extend(['-o %s' % (str(target[0]))])
	args.extend(utilities.build_includes(arch.includes()))
	args.extend(utilities.build_defines(arch.property('defines', default={})))
	args.extend(arch.property('cflags', default=[]))

	return SCons.Action.Action(" ".join(args), 'Compiling %s' % str(source[0]))

_xc16_obj = SCons.Builder.Builder(
	generator = xc16_generator,
	suffix='.o',
	source_scanner = SCons.Scanner.C.CScanner())

def generate(env):
	env['BUILDERS']['xc16_gcc'] = _xc16_obj

def exists(env):
	return 1


#project.py
#Create a FIRMWARE project for Proteus VSM 8.0.

import fnmatch
import os
import shutil
import sys
import tempfile
import zipfile
import xml.etree.ElementTree as ET
from bs4 import BeautifulSoup
from pymomo.utilities import build, paths

Extensions = ['c', 'asm', 'h', 'inc', 's']

asm_options = []
asm_options.append(['-g'])
asm_options.append(['--debug'])
asm_options.append(['-omf=', 'coff'])
asm_options.append(['-mcpu=', '24F16KA101'])

class Project:
	"""
	Given a directory full of PIC files, create a proteus vsm project. 
	"""

	def __init__(self, dirs, type, template):
		self.dirs = dirs
		self.files = []
		self.template = template

		self.dirs.extend(self._load_shared_dirs(type))

		for d in self.dirs:
			self.files.extend(self._recursive_glob(d, Extensions))

		self._get_name()

	def create(self, out):
		self._make_structure()
		self._copy_files()
		self._copy_template()
		self._build_firmware_xml()
		self._zip_project(out)

	def _file_name(self, f):
		"""
		Given a file name in self.files, return the name that should go in FIRMWARE.XML
		This is the base file name with the project directory prepended.
		"""

		name = os.path.basename(f)

		return "/".join([self.name, name])

	def _add_to_ld(self, ldtag, file, tool):
		new_f = ET.SubElement(ldtag, 'FILE')
		new_f.set('TOOL', tool)

		fname = ET.SubElement(new_f, 'FILE')
		fname.set('NAME', file)

	def _add_option(self, conf_tag, tool, name, value):
		new_o = ET.SubElement(conf_tag, 'OPTION')
		new_o.set('TOOL', tool)
		new_o.set('NAME', name)
		new_o.set('VALUE', value)

	def _configure_asm(self, conf_tag):
		for conf in asm_options:
			if len(conf) == 1:
				conf.append('')

			self._add_option(conf_tag, 'ASM', *conf)

	def _build_firmware_xml(self):
		z = zipfile.ZipFile(self.template, "r")

		with z.open('FIRMWARE/%s.XML' % self.name, "r") as f:
			tree = ET.parse(f)
			root = tree.getroot()

			files = root.find('FILES')

			ld_tags = root.findall(".//FILE[@TOOL='LD']")
			conf_tags = root.findall("./CONFIGURATION")

			#Make sure we add the optimize -O1 flag so that mainboard code
			#compiles (it's too big otherwise)
			for tag in conf_tags:
				self._add_option(tag, 'CC', '', '-O1')
				self._configure_asm(tag)

			#Remove all of the old files from this list
			for elem in files:
				files.remove(elem)

			new_files = [self._file_name(x) for x in self.files]

			#The new version 8.1 of proteus requires the files be listed in the compilation
			#step explicitly, rather than rebuilding it itself if we don't tell it anything.
			for tag in ld_tags:
				for filetag in filter(lambda x: x.tag == 'FILE', tag):
					tag.remove(filetag)

				for f in fnmatch.filter(new_files, '*.c'):
					self._add_to_ld(tag, f, tool='CC')

				for f in fnmatch.filter(new_files, "*.s"):
					self._add_to_ld(tag, f, tool='ASM')

			#Attach Files to the Project for Browsing
			for f in new_files:
				new_f = ET.SubElement(files, 'FILE')
				new_f.set('ATTACH', '1')

				name, ext = os.path.splitext(f)

				if ext == ".h":
					group = 'Header Files'
				else:
					group = "Source Files"

				new_f.set('GROUP', group)
				new_f.set('NAME', f)

			tree.write(os.path.join(self.firm, '%s.XML' % self.name))

	def _get_name(self):
		print "Using Template Project: %s" % os.path.basename(self.template)

		z = zipfile.ZipFile(self.template, "r")
		with z.open('FIRMWARE.XML') as firmfile:
			f = BeautifulSoup(firmfile)

			projs = f.firmware.projects
			names = []

			for p in projs.find_all('string'):
				names.append(p.string.strip())

			if len(names) == 0:
				raise ValueError("Could not extract name from Proteus VSM project")
			elif len(names) > 1:
				raise ValueError("Multiple firmware projects in Proteus VSM project: %s" % str(names))

			print "Found firmware project named: %s" % names[0]

			self.name = names[0] 

	def _copy_template(self):
		"""
		Copy specific files from the template to our version.
		"""

		z = zipfile.ZipFile(self.template, "r")
		z.extract('ROOT.CDB', self.basedir)
		z.extract('ROOT.DSN', self.basedir)
		z.extract('PROJECT.XML', self.basedir)
		z.extract('FIRMWARE.XML', self.basedir)

		#Extract all files in SCRIPTS directory
		for f in z.namelist():
			if "SCRIPTS" in f:
				z.extract(f, self.basedir)


	def _copy_files(self):
		"""
		Copy all of the files into the project directory
		"""

		if not os.path.isdir(self.proj):
			os.makedirs(self.proj)

		for path in self.files:
			shutil.copy2(path, self.proj)

	def _make_structure(self):
		basedir = tempfile.mkdtemp()
		firmdir = os.path.join(basedir, 'FIRMWARE')
		projdir = os.path.join(firmdir, self.name.upper())
		scriptsdir = os.path.join(basedir, 'SCRIPTS')

		os.makedirs(firmdir)
		os.makedirs(projdir)
		os.makedirs(scriptsdir)

		self.basedir = basedir
		self.scripts = scriptsdir
		self.firm = firmdir
		self.proj = projdir

	def _zip_project(self, zipname):
		shutil.make_archive(base_name=zipname, root_dir=self.basedir, base_dir='.', format='zip')

		dirs, filename = os.path.split(zipname)
		shutil.move(zipname + '.zip', os.path.join(dirs, zipname + ".pdsprj"))

	def _load_shared_dirs(self, type):
		"""
		Given the type of project that we're creating (either pic12 or pic24),
		load in the appropriate shared directories.
		"""

		if type not in ["pic12", "pic24"]:
			raise ValueError("Trying to create an unsupported proteus project: type=%s" % type)

		momo = paths.MomoPaths()
		config = build.load_settings()


		port = [os.path.join(momo.base, f) for f in config["reference"]["portable_dirs"]]
		specific = [os.path.join(momo.base, f) for f in config["reference"]["%s_dirs" % type]]

		return port + specific

	def _recursive_glob(self, folder, extensions):
		matches = []

		for root, dirnames, filenames in os.walk(folder):
			for ext in extensions:
				files = fnmatch.filter(filenames, '*.%s' % ext)
				matches.extend([os.path.join(root, f) for f in files])

		return matches


#log.py
#Parse the log file from a gpsim test run and convert it to a series of events

import re
import statements
from statements.unknown import UnknownStatement
from pymomo.hex8 import symbols

class LogFile:	
	def __init__(self, filename, symtab=None):
		with open(filename, "r") as f:
			lines = f.readlines()

		self.entries = []

		#Only keep entries where we write to the logging registers
		lines = filter(lambda x: not x.startswith('  Read:'), lines)
		lines = filter(lambda x: not x.startswith('  BREAK:'), lines)

		if len(lines) % 2 != 0:
			raise ValueError("File length is invalid, filtered entries should be a multiple of 2, len=%d." % len(lines))

		entries = zip(lines[0::2], lines[1::2])
		entries = map(lambda x: (x[0]+x[1]).rstrip(), entries)
		entries = map(lambda x: re.split('\W+', x), entries)
		info = map(extract_info, entries)
		info = info[:-1] #chomp last entry which is always a duplicate of test ended

		#find control entries
		controls = filter(lambda x: x[1]['dest'] == 'ccpr1l', enumerate(info))
		controls = map(lambda x: x[0], controls)
		lengths = [x-controls[i] for i,x in enumerate(controls[1:])]

		lengths.append(1) #final control statement has no data entries

		if len(lengths) != len(controls):
			raise ValueError("Logic error in log file handling, control statements were not matched with data statements")

		#at this point we have the indices corresponding to all control statements
		#and their lengths build the statements
		self.symtab = symtab

		#initialize program counter
		self.current_address = 0

		statements = [{'control': info[c], 'data':info[c+1:c+lengths[i]]} for i,c in enumerate(controls)]
		entries = map(lambda x:self._process_statement(x), statements)
		
		self.entries = filter(lambda x:x.keep() == True, entries)

	def _process_statement(self, statement):
		if statement['control']['value'] in statements.statements:
			return statements.statements[statement['control']['value']](statement, self)

		return UnknownStatement(statement, self)

	def pretty_print(self, color=True):
	    for entry in self.entries:
	    	print entry.format_line(self.symtab, use_colors=color)

	def save(self, path):
		with open(path, "w") as f:
			for entry in self.entries:
				f.write(entry.format_line(self.symtab, use_colors=False) + '\n')

	def test_passed(self, testcase):
		for entry in self.entries:
			if entry.error():
				return False

		#Make sure that all checkpoints were passed and logged the correct values
		pts = testcase.checkpoints

		passed_pts = [x for x in self.entries if isinstance(x, statements.logcheckpoint.LogCheckpoint)]

		if len(pts) != len(passed_pts):
			return False

		#Make sure the symbol and value match for each passed checkpoint
		for expected, passed in zip(pts, passed_pts):
			passed_sym = self.symtab.map_address(passed.address)[0]

			if passed_sym != expected[0]:
				return False
			if expected[1] != passed.data:
				return False

		return True

def extract_info(entry):
	info = {}

	if len(entry) != 13:
		raise ValueError("Invalid entry had illegal length %d" % len(entry))

	info['cycle'] = int(entry[0],0)
	info['proc'] = entry[1]
	info['value'] = int(entry[7],0)
	info['dest'] = entry[9]

	return info


#pic12

from SCons.Script import *
from SCons.Environment import Environment
import os
import fnmatch
import json as json
import sys
import os.path
from copy import deepcopy
import utilities

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from pymomo.utilities import build
from pymomo.mib.config12 import MIB12Processor


def configure_env_for_xc8(env, **kwargs):
	"""
	Setup all of the environmental varibles that the xc8 Builder needs in order
	to property configure and build an 8-bit module.  There's a lot of sensitve
	magic here!  Getting the RAM and ROM ranges wrong will cause very subtle but
	deadly bugs as the executive and applications begin to overwite each other's
	variables.  Be careful when you modify this function.
	"""
	arch = env['ARCH']
	proc = MIB12Processor('No Name', arch.settings)

	#concatenate cflags
	flags = deepcopy(arch.property('xc8_flags', []))
	flags.extend(arch.property('extra_xc8_flags', []))

	#set up definitions for this chip
	defines = deepcopy(arch.property('defines'))
	defines["kFirstApplicationRow"] = proc.first_app_page
	defines["kFlashRowSize"] = proc.row_size
	defines["kFlashMemorySize"] = proc.total_prog_mem
	
	#set up includes
	incs = arch.includes()

	#set up generic environment variables that are set for both app and exec
	env['MIB_API_BASE'] = proc.api_range[0]
	env['CHIP'] = proc

	#Let the module type dictate whether we build for app or exec
	if arch.property('is_exec'):
		env['ROMSTART'] = proc.exec_rom[0]
		env['ROMEND'] = proc.exec_rom[1]

		flags += ['-L-Pmibapi=%xh' % env['MIB_API_BASE'], '-L-Papp_vectors=%xh' % proc.app_rom[0]] #Place the MIB api in the right place

		env['RAMEXCLUDE'] =  arch.property('application_ram')
	else:
		exec_range = proc.exec_rom

		env['ROMSTART'] = exec_range[1]+1
		env['ROMEND'] = proc.total_prog_mem - 1  

		#Make sure we don't let the code overlap with the MIB map in high memory
		env['ROMEXCLUDE'] = []

		flags += ['-L-preset_vec=%xh' % (exec_range[1]+1)]

		mibstart = proc.mib_range[0]
		lnk_cmd = '-L-Pmibblock=%xh' % mibstart
		flags += [lnk_cmd]

		#MIB12 Executive takes all ram in first 
		env['RAMEXCLUDE'] = arch.property('executive_ram')
		env['NO_STARTUP'] = True

	env['XC8FLAGS'] = flags
	env['XC8FLAGS'] += utilities.build_defines(defines)
	env['INCLUDE'] = incs

def build_module(name, arch):
	"""
	Configure Scons to build an application module for the 8bit pic microchip archicture indicated in arch_name.
	"""

	dirs = arch.build_dirs()

	builddir = dirs['build']
	VariantDir(builddir, 'src', duplicate=0)

	env = Environment(tools=['xc8_compiler', 'patch_mib12', 'xc8_symbols'], ENV = os.environ)
	env.AppendENVPath('PATH','../../tools/scripts')
	env.AppendENVPath('PATH','../../tools/bin')
	env['ARCH'] = arch
	env['MODULE'] = name

	#Load in all of the xc8 configuration from build_settings
	configure_env_for_xc8(env)
	compile_mib(env)

	Export('env')
	SConscript(os.path.join(builddir, 'SConscript'))

	#Compile the *.mib file specified in env['MIBFILE'] in SConscript into a command_map.asm file

	prods = [os.path.join(dirs['build'], 'mib12_app_module.hex'), os.path.join(dirs['build'], 'mib12_app_module_symbols.h'), os.path.join(dirs['build'], 'mib12_app_module_symbols.stb')]

	hexfile = env.InstallAs(os.path.join(dirs['output'], '%s_%s.hex' % (name, arch.arch_name())), prods[0])
	symheader = env.InstallAs(os.path.join(dirs['output'], '%s_symbols_%s.h' % (name, arch.arch_name())), prods[1])
	symtable = env.InstallAs(os.path.join(dirs['output'], '%s_symbols_%s.stb' % (name, arch.arch_name())), prods[2])

def compile_mib(env):
	"""
	Given a path to a *.mib file, use mibtool to process it and return a command_map.asm file
	return the path to that file.
	"""

	dirs = env["ARCH"].build_dirs()
	mibname = os.path.join('src', 'mib', env["MODULE"] + ".mib")
	cmdmap_path = os.path.join(dirs['build'], 'command_map.asm')

	env['MIBFILE'] = '#' + cmdmap_path

	return env.Command(cmdmap_path, mibname, 'momo-mib gen -o %s $SOURCE' % dirs['build'])



#unit_test12.py

import unit_test
import pic12_unit
from pymomo.exceptions import *
from pymomo.gpysim import i2c_log
import os.path

typemap = { 'RS': i2c_log.RepeatedStartCondition, 'S': i2c_log.StartCondition,
			'P': i2c_log.StopCondition, 'A': i2c_log.AddressByte, 'D': i2c_log.DataByte}

class Pic12UnitTest (unit_test.UnitTest):
	def __init__(self, files, **kwargs):
		#Must initialize variables before calling super init b/c they will
		#be overwritten by imported header information.
		self.i2c_capture = None
		self.checkpoints = []
		unit_test.UnitTest.__init__(self, files, **kwargs)

	#PIC12 Specific Attibutes
	def _parse_checkpoints(self, value):
		pts = value.split(',')
		ptlist = [self._add_checkpoint(x) for x in pts]
		self.checkpoints = ptlist

	def _parse_i2c_capture(self, value):
		"""
		I2C Capture: <sequence> allows unit tests to specify what I2C signals must 
		be transmitted exactly in order for the test to pass.  It is in the form of
		<signal 1>[, <signal 2>, ...]
		where signal is one of the following:
		S, RS, P, number/[A|N], address/[R|W][A|N]
		S, RS and P correspond with start, repeated start and stop conditions.
		number should be an 8-bit integer literal in decimal or prefixed hex or
		binary.
		address should be a 7-bit literal with R or W corresponding to read or
		write indication.
		For numbers and addresses, A corresponds to the acknowledge bit being set
		and N corresponds to it being cleared.
		"""

		sigs = [x.strip() for x in value.split(',')]
		self.i2c_capture = [self._process_i2c_signal(x) for x in sigs]
		self.add_intermediate('clock.txt', folder='test')
		self.add_intermediate('data.txt', folder='test')

	def _process_i2c_signal(self, sig):
		if sig.upper() == 'S':
			return ('S', None)
		elif sig.upper() == 'RS':
			return ('RS', None)
		elif sig.upper() == 'P':
			return ('P', None)
		else:
			#sig should be a number
			splitvals = sig.split('/')
			if len(splitvals) != 2:
				raise BuildError("No modifier specified in data byte", test=self.name, value=sig)

			val = int(splitvals[0],0)
			mods = splitvals[1].upper()

			if len(mods) == 0 or len(mods) > 2:
				raise BuildError("Modifier too long or missing in I2C capture data byte specification", test=self.name, value=sig, parsed_modifier=mods)

			if mods[-1] != 'A' and mods[-1] != 'N':
				raise BuildError("Invalid A/N modifier in I2C capture data or address specification", test=self.name, value=sig, parsed_modifier=mods)

			if len(mods) == 2 and mods[0] != 'R' and mods[0] != 'W':
				raise BuildError("Invalid R/W modifier in I2C capture address byte specification", test=self.name, value=sig, parsed_modifier=mods)

			#Parse a data byte
			if len(mods) == 1:
				return ('D', val, mods[0] == 'A')
			else:
				return ('A', val, mods[0] == 'W', mods[1] == 'A')

	def _parse_additional(self, value):
		"""
		Parse an Additional:<file1>,<fileN>
		header line, taking the appropriate action
		"""

		parsed = value.split(',')
		files = map(lambda x: x.rstrip().lstrip(), parsed)

		for f in files:
			base, ext = os.path.splitext(f)

			if ext == '.as':
				self.files += (os.path.join(self.basedir, f), )
			elif ext == '.cmd':
				self.cmdfile = os.path.join(self.basedir, f)
			else:
				raise BuildError("Unknown file extension in Additional header", test=self.name, filename=f)

	def _parse_target(self, target):
		return target

	def build_target(self, target, summary_env):
		cmdfile = None

		if hasattr(self, 'cmdfile'):
			cmdfile = self.find_support_file(self.cmdfile, target)
		
		pic12_unit.build_unittest(self, target, summary_env, cmds=cmdfile)

	def check_output(self, chip, log):
		with open(log, 'a') as f:
			f.write('\n## Additional Tests ##\n')

			if self.i2c_capture is not None:
				passed, observed, expected = self._check_i2c_output(chip)
				if passed:
					f.write("I2C Output Match: PASSED\n")
				else:
					f.write("I2C Output Match: FAILED\n")
					f.write("# Observed Output #\n")
					f.write(observed)
					if observed[-1] != '\n':
						f.write('\n')
					f.write("# Expected Output #\n")
					f.write(expected)
					if expected[-1] != '\n':
						f.write('\n')

					return passed

		return True

	def _format_i2c_sequence(self):
		events = [self._format_event(x) for x in self.i2c_capture]
		return "\n".join(events)

	def _format_start(self, ev):
		return "Start"

	def _format_repeated_start(self, ev):
		return "Repeated Start"

	def _format_stop(self, ev):
		return "Stop"
	
	def _format_data(self, ev):
		a = 'NACK'
		if ev[2]:
			a = 'ACK'

		return "0x%x %s" % (ev[1], a)

	def _format_address(self, ev):
		a = 'NACK'
		if ev[3]:
			a = 'ACK'

		d = 'READ from'
		if ev[2]:
			d = 'WRITE to'

		return "%s 0x%x %s" % (d, ev[1], a)

	def _format_event(self, ev):
		if ev[0] == 'S':
			return self._format_start(ev)
		elif ev[0] == 'P':
			return self._format_stop(ev)
		elif ev[0] == 'D':
			return self._format_data(ev)
		elif ev[0] == 'A':
			return self._format_address(ev)
		elif ev[0] == 'RS':
			return self._format_repeated_start(ev)
		
		raise InternalError("Unknown event type in i2c log, cannot format", event=ev)

	def _check_i2c_output(self, chip):
		"""
		Verify that the unit test's expected i2c output actually matches what is output during
		the simulation.
		"""

		folder = self.build_dirs(chip)['test']
		datapath = os.path.join(folder, 'data.txt')
		clockpath = os.path.join(folder, 'clock.txt')

		analyzer = i2c_log.I2CAnalyzer.FromGPSIMLogs(clockpath, datapath)

		if len(analyzer.bus_events) != len(self.i2c_capture):
			return False, analyzer.format(), self._format_i2c_sequence()

		status = True

		for exp, act in zip(self.i2c_capture, analyzer.bus_events):
			expected_type = typemap[exp[0]]
			if not isinstance(act, expected_type):
				status = False
				break

			if exp[0] == 'D':
				if exp[1] != act.value or exp[2] != act.acked:
					status = False
					break

			if exp[0] == 'A':
				if exp[2] != act.is_write or exp[1] != act.address or exp[3] != act.acked:
					status = False
					break

		return status, analyzer.format(), self._format_i2c_sequence()


	def _add_checkpoint(self, pt):
		"""
		Given a checkpoint of the form symbol=value or symbol, parse the symbol and
		value.  If no value is given, value=0 is assumed
		"""

		if '=' in pt:
			symbol, val = pt.split('=')
			symbol = symbol.strip()
			val = val.strip()
			val = int(val,0)
			return symbol, val
		
		return pt.strip(), 0

unit_test.known_types['executive'] = Pic12UnitTest
unit_test.known_types['application'] = Pic12UnitTest
unit_test.known_types['executive_integration'] = Pic12UnitTest



#logical_part.py

import re
from pymomo.pcb import reference

def convert(arg, **kwargs):
	if arg is None:
		return None

	if isinstance(arg, Part):
		return arg

	elif isinstance(arg, dict):
		return Part(**arg)

	raise ValueError("Creating a logical_part from any type other than a Part object is not supported")

def validate_matchable(arg, **kwargs):
	if not arg.matchable():
		raise ValueError("part does not have enough metadata to be matchable")

def default_formatter(arg, **kwargs):
	return str(arg)

class Part:
	"""
	An electronic component.
	"""

	ref = reference.PCBReferenceLibrary()

	def __init__(self, name, package, placement, mpn=None, manu=None, dist=None, distpn=None, value=None, desc=None):
		"""
		Create a part object from the data passed
		"""

		self.name = name
		self.package = package
		self.mpn = mpn
		self.manu = manu
		self.value = value
		self.distpn = distpn
		self.dist = dist
		self.placement = placement
		self.desc = desc

		#If no description is given try creating a generic one based on the type of the part (resistor, etc)
		if self.desc is None:
			self.desc = Part.ref.find_description(self.name, self.value)

	def matchable(self):
		"""
		Return true if this part has enough information to be matched and priced online
		"""

		return self.name != None and ((self.manu != None and self.mpn != None) or (self.dist != None and self.distpn != None))

	def placeable(self):
		"""
		Return true if this part has enough information to be physically located on a board
		"""

		return (self.package != None and self.placement != None)

	def complete(self):
		"""
		Return true if this part has all required information for BOM matching and placing
		"""

		return self.matchable() and self.placeable()

	def unique_id(self):
		"""
		Return a unique key that can be used to group multiple parts that are identical.
		"""

		if self.manu and self.mpn:
			return "%s_%s" % (self._canonicalize(self.manu), self._canonicalize(self.mpn))

		return "%s_%s" % (self._canonicalize(self.dist), self._canonicalize(self.distpn))

	def _canonicalize(self, string):
		"""
		Given a string, remove all spaces, - and _ and make it lower case.
		"""

		s = string.lower()
		return s.replace(' ', '').replace('-','').replace('_','')

	def __str__(self):
		string = ""
		string += "Part %s" % str(self.name) + '\n'
		string += " Manufacturer: " + str(self.manu) + '\n'
		string += " MPN: " + str(self.mpn) + '\n'
		string += " Distributor: " + str(self.dist) + '\n'
		string += " Distributor PN: " + str(self.distpn) + '\n'
		string += " Package: " + str(self.package) + '\n'
		string += " Location: " + str(self.placement) + '\n'
		string += " Description: " + str(self.desc)
		
		return string

	def __eq__(self, other):
		try:
			return self.name == other.name and self.package == other.package and self.mpn == other.mpn and \
			self.manu == other.manu and self.value == other.value and self.distpn == other.distpn and \
			self.dist == other.dist and self.placement == other.placement and self.desc == other.desc
		except AttributeError:
			raise False



#controllerblock.py
from pymomo.utilities.typedargs.types import *
from pymomo.exceptions import *
from pymomo.utilities.typedargs.annotate import *
from hexfile import HexFile

def print_validity(info):
	print "Validating Controller Metadata Block"

	magic = info['magic']
	if magic[0] == True:
		print "+ Valid magic number (0xBAAD, 0xDAAD)"
	else:
		print "- Invalid magic number: (0x%x, 0x%x)" % (magic[0], magic[1])

	firmware_length = info['firmware_end']
	if firmware_length[0] == True:
		print "+ Valid firmware length (0x%x)" % (firmware_length[1],)
	else:
		print "- Invalid firmware length (stored=0x%x, wanted=0x%x)" % (firmware_length[1], firmware_length[2])

	ivt = info['ivt_checksum']
	if ivt[0] == True:
		print "+ IVT checksum matches (0x%x)" % ivt[1]
	else:
		print "- IVT checksum mismatch (stored=0x%x, calculated=0x%x)" % (ivt[2], ivt[1])

	aivt = info['aivt_checksum']
	if aivt[0] == True:
		print "+ AIVT checksum matches (0x%x)" % aivt[1]
	else:
		print "- AIVT checksum mismatch (stored=0x%x, calculated=0x%x)" % (aivt[2], aivt[1])

	code = info['code_checksum']
	if code[0] == True:
		print "+ Program code checksum matches (0x%x)" % code[1]
	else:
		print "- Program code checksum mismatch (stored=0x%x, calculated=0x%x)" % (code[2], code[1])

	block = info['block_checksum']
	if block[0] == True:
		print "+ Metadata block checksum matches (0x%x)" % block[1]
	else:
		print "- Metadata block checksum mismatch (stored=0x%x, calculated=0x%x)" % (block[2], block[1])

	if info['valid']:
		print "\n**Metadata Block is valid**\n"
	else:
		print "\n**Metadata Block is invalid**\n"


@context("ControllerBlock")
class ControllerBlock:
	"""
	The metadata block in a pic24 controller firmware image.  This includes
	data like the specific hardware version that the controller is compiled
	for as well as checksums for various parts of the image that allow the
	controller board's bootloader to verify that the firmware image is valid
	and complete.
	"""

	firmware_length = 0xA600

	@param("source", "path", "readable", desc="hex file to load")
	def __init__(self, source):
		self.hex = HexFile(source, 24, 4, 2)
		self._parse_block()

	def _calculate_checksum(self, start, length):
		checksum = 0

		for i in xrange(start, start+length, 2):
			checksum += ((self.hex[i] >> 16) + self.hex[i])

		return ((~checksum) + 1) & 0xFFFF

	def _parse_block(self):
		base = 0x200
		self.magic = (self.hex[base+0], self.hex[base+2])
		self.reset = (self.hex[base+4], self.hex[base+6])
		self.firmware_end = self.hex[base+8]

		self.hw_version = ""
		for i in xrange(10, 10+8, 2):
			val = self.hex[base+i]
			low = val & 0xFF
			high = val >> 8 & 0xFF
			self.hw_version += chr(low)
			self.hw_version += chr(high)

		self.ivt_checksum = self.hex[base+18]
		self.aivt_checksum = self.hex[base+20]
		self.code_checksum = self.hex[base+22]
		self.block_checksum = self.hex[base+30]

	def _write_block(self):
		base = 0x200
		self.hex[base+0] = self.magic[0]
		self.hex[base+2] = self.magic[1]
		
		self.hex[base+4] = self.reset[0]
		self.hex[base+6] = self.reset[1]

		self.hex[base+8] = self.firmware_end

		for i in xrange(0, 8, 2):
			if i < (len(self.hw_version)-1):
				low = ord(self.hw_version[ i+ 0])
				high = ord(self.hw_version[i+ 1])
			elif i < len(self.hw_version):
				low = ord(self.hw_version[i])
				high = ord(' ')
			else:
				low = ord(' ')
				high = ord(' ')

			word = ((high & 0xFF) << 8) | (low & 0xFF)
			self.hex[base + 10 + i] = word

		self.hex[base+18] = self.ivt_checksum
		self.hex[base+20] = self.aivt_checksum
		self.hex[base+22] = self.code_checksum
		for i in xrange(24, 30, 2):
			self.hex[base+i] = 0

		self.hex[base+30] = self._calculate_checksum(0x200, 30)

	@param("dest", "path", "writeable", desc="output hex file path")
	def save(self, dest):
		"""
		Save the hexfile with a potentially updated metadata block
		into the path specified by dest.
		"""

		self._write_block()
		self.hex.save(dest)

	@annotated
	def update_checksums(self):
		"""
		Calculate the correct checksums that should be stored in the metadata block
		and add them to the in-memory copy of the block.
		"""

		self.ivt_checksum 	= self._calculate_checksum(0x04, 0x100 - 0x04)
		self.aivt_checksum	= self._calculate_checksum(0x100, 0x100)
		self.code_checksum	= self._calculate_checksum(0x200 + 32, ControllerBlock.firmware_length - 32)
		self.block_checksum = self._calculate_checksum(0x200, 30)

	@returns('HW Compatibility Version')
	def get_hw_version(self):
		return self.hw_version

	@param("hw", "string", desc="a string up to 16 bytes long")
	def set_hw_version(self, hw):
		"""
		Set the embedded hw compatibility string for this metadata block.  This is an
		ASCII string up to 16 bytes in length not including the terminating NULL.  It 
		is checked upon bootup against the hardcoded hw_version stored in the mainboard
		bootloader and if it does not match the controller hex file is not executed.
		"""

		if len(hw) > 16:
			hw = hw[:16]

		self.hw_version = hw

	@returns("Controller Block Status", printer=print_validity)
	def validate(self):
		"""
		Validate the data contained in this metadata block.
		"""

		status = {}
		status['magic'] = self._validate_magic()
		status['firmware_end'] 	= self._validate_firmware_length()
		status['ivt_checksum'] 	= self._validate_firmware_region(0x04, 0x100 - 0x04, self.ivt_checksum)
		status['aivt_checksum']	= self._validate_firmware_region(0x100, 0x100, self.aivt_checksum)
		status['code_checksum']	= self._validate_firmware_region(0x200 + 32, ControllerBlock.firmware_length - 32, self.code_checksum)
		status['block_checksum']= self._validate_firmware_region(0x200, 30, self.block_checksum)

		valid = reduce(lambda x,y: x and y, map(lambda x: x[0], status.itervalues()), True)
		status['valid'] = valid
		return status

	def _validate_magic(self):
		if self.magic[0] == 0xBAAD and self.magic[1] == 0xDAAD:
			return (True, self.magic[0], self.magic[1])
		else:
			return (False, self.magic[0], self.magic[1])

	def _validate_firmware_length(self):
		if self.firmware_end == ControllerBlock.firmware_length:
			return (True, self.firmware_end)

		return (False, self.firmware_end, ControllerBlock.firmware_length)

	def _validate_firmware_region(self, start, length, comp):
		check = self._calculate_checksum(start, length)
		if comp == check:
			return (True, check, comp)

		return (False, check, comp)




#decode.py
#Simple routines for decoding information from pic12 instructions

def decode_goto(ih, addr):
	"""
	Return the address pointed to by the goto instruction at addr.
	Raises an exception if addr is not a goto instruction
	"""

	val = ih[addr]

	if not match_high(val, 0b101, 3):
		raise ValueError("Instruction at 0x%X was not a goto." % addr) 

	return val & ((1 << 11)-1)

def decode_retlw(ih, addr):
	val = ih[addr]

	if not match_high(val, 0b110100, 6):
		raise ValueError("Instruction at 0x%X was not a retlw instruction" % addr)

	return val & 0xFF

def decode_brw(ih, addr):
	val = ih[addr]

	if val != 0b1011:
		raise ValueError("Instruction at 0x%X was not a brw instruction" % addr)

	return True

def decode_string(ih, addr, length):
	string = ""

	for i in xrange(0, length):
		a = addr + i
		string += chr(ih[a] & 0xFF)

	return string 

def decode_table(ih, addr, mapper):
	"""
	Given a jumptable of the form:
	addr:
	BRW
	<instr>
	<instr>
	<instr>
	...

	return an array with mapper(<instr>) values.  Decoding stops after 256 entries
	or when mapper throws an exception
	"""

	decode_brw(ih, addr)

	tab = []

	#Table can be at most 256 entries long
	for i in xrange(0, 256):
		addr += 1
		try:
			tab.append(mapper(ih, addr))
		except ValueError:
			break

	return tab

def match_high(val, mask, num_bits):
	"""
	Return true if the high num_bits bits of val equal mask
	"""

	shift = 14-num_bits
	if shift < 0:
		raise ValueError("num_bits must be >= 0 in match_high, was %d" % num_bits)
	elif shift == 14:
		return True

	val = val >> shift

	return (val == mask)